@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
 #include "config.h"
@@ -54,9 +54,9 @@ _clear_sectors (PedGeometry* geom, PedSector start, PedSector count)
 
 	for (pos = start;
 	     pos < start + count;
-	     pos += CLEAR_BUFFER_SIZE, to_go -= CLEAR_BUFFER_SIZE) {
+	     pos += CLEAR_BUFFER_SECTORS, to_go -= CLEAR_BUFFER_SECTORS) {
 		if (!ped_geometry_write (geom, buffer, start,
-					 PED_MIN (CLEAR_BUFFER_SIZE, to_go)))
+					 PED_MIN (CLEAR_BUFFER_SECTORS, to_go)))
 			return 0;
 	}
 
@@ -102,7 +102,9 @@ _clear_partial_range (PedGeometry* geom, PedSector sector, int start, int end)
 static int
 _clear_clusters (PedFileSystem* fs, FatCluster start, FatCluster count)
 {
-	return fat_write_clusters (fs, buffer, start, count);
+	FatSpecific*	fs_info = FAT_SPECIFIC (fs);
+	return _clear_sectors (fs->geom, fat_cluster_to_sector(fs, start),
+			       count * fs_info->cluster_sectors);
 }
 
 /* FAT code ******************************************************************/
@@ -246,7 +248,7 @@ main (int argc, char* argv[])
 		goto error_destroy_disk;
 
 	if (strncmp (fs->type->name, "fat", 3)) {
-		printf ("Not a FAT filesystem!\n");
+		printf ("Not a FAT file system!\n");
 		goto error_close_fs;
 	}
 
