@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
 #ifndef PED_DISK_H_INCLUDED
@@ -49,10 +49,11 @@ typedef enum {
 	PED_PARTITION_LBA=7,
 	PED_PARTITION_HPSERVICE=8,
 	PED_PARTITION_PALO=9,
-	PED_PARTITION_PREP=10
+	PED_PARTITION_PREP=10,
+	PED_PARTITION_MSFT_RESERVED=11
 } PedPartitionFlag;
 #define PED_PARTITION_FIRST_FLAG	PED_PARTITION_BOOT
-#define PED_PARTITION_LAST_FLAG		PED_PARTITION_PREP
+#define PED_PARTITION_LAST_FLAG		PED_PARTITION_MSFT_RESERVED
 
 typedef enum {
 	PED_DISK_TYPE_EXTENDED=1,	/* supports extended partitions */
