@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
 #ifndef PED_EXCEPTION_H_INCLUDED
@@ -71,6 +71,7 @@ extern char* ped_exception_get_type_string (PedExceptionType ex_type);
 extern char* ped_exception_get_option_string (PedExceptionOption ex_opt);
 
 extern void ped_exception_set_handler (PedExceptionHandler* handler);
+extern PedExceptionOption ped_exception_default_handler (PedException* ex);
 
 extern PedExceptionOption	ped_exception_throw (PedExceptionType ex_type,
 						     PedExceptionOption ex_opt,
