@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
 #ifndef PARTED_H_INCLUDED
@@ -32,6 +32,7 @@ typedef struct _PedArchitecture PedArchitecture;
 #include <parted/exception.h>
 #include <parted/filesys.h>
 #include <parted/natmath.h>
+#include <parted/unit.h>
 
 #include <stdlib.h>
 
