@@ -1,3 +1,14 @@
+/* This file contains libparted's constraint solver interface.  The constraint
+ * solver allows you to specify constraints on where a partition or file system
+ * (or any PedGeometry) may be placed/resized/etc.  For example, you might want
+ * to make sure that a file system is at least 10 Gb, or that it starts at the
+ * beginning of new cylinder.
+ * 
+ * The constraint solver in this file unifies solver in geom.c (which allows
+ * you to specify constraints on ranges) and natmath.c (which allows you
+ * to specify alignment constraints).
+ */
+
 /*
     libparted - a library for manipulating disk partitions
     Copyright (C) 2000, 2001 Free Software Foundation, Inc.
@@ -14,14 +25,12 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
 #include <parted/parted.h>
 #include <parted/debug.h>
 
-static const PedConstraint* ped_constraint_none = NULL;
-
 int
 ped_constraint_init (
 	PedConstraint* constraint,
@@ -73,6 +82,9 @@ error:
 	return NULL;
 }
 
+/* This function returns a constraint that requires a region to be
+ * entirely contained inside "max", and to entirely contain "min".
+ */
 PedConstraint*
 ped_constraint_new_from_min_max (
 	const PedGeometry* min,
@@ -96,6 +108,9 @@ ped_constraint_new_from_min_max (
 			min->length, max->length);
 }
 
+/* This function returns a constraint that requires a region to entirely
+ * contain "min".
+ */
 PedConstraint*
 ped_constraint_new_from_min (const PedGeometry* min)
 {
@@ -107,6 +122,9 @@ ped_constraint_new_from_min (const PedGeometry* min)
 	return ped_constraint_new_from_min_max (min, &full_dev);
 }
 
+/* This function returns a constraint that requires a region to lie
+ * entirely within "max".
+ */
 PedConstraint*
 ped_constraint_new_from_max (const PedGeometry* max)
 {
@@ -131,6 +149,10 @@ ped_constraint_duplicate (const PedConstraint* constraint)
 		constraint->max_size);
 }
 
+/* This function returns a constraint that requires a region to
+ * satisfy both a and b.  Moreover, any region satisfying a and b
+ * will also satisfy the returned constraint.
+ */
 PedConstraint*
 ped_constraint_intersect (const PedConstraint* a, const PedConstraint* b)
 {
@@ -204,11 +226,10 @@ ped_constraint_destroy (PedConstraint* constraint)
 	}
 }
 
-/* constraint->start_range isn't "the end of the story", wrt where
- * the start can lie.  We also need to sure start lies in
- * [end_range->start - max_size + 1, end_range->end - min_size + 1]
- * 	This function returns a "canonical" geometry.  All sectors
- * in this geometry that also satisfy alignment requirements have
+/* This function returns the region within which the start must lie
+ * in order to satisfy a constriant.  It takes into account
+ * constraint->start_range, constraint->min_size and constraint->max_size.
+ * All sectors in this range that also satisfy alignment requirements have
  * an end, such that the (start, end) satisfy the constraint.
  */
 static PedGeometry*
@@ -250,8 +271,8 @@ _constraint_get_canonical_start_range (const PedConstraint* constraint)
 				       constraint->start_range);
 }
 
-/* returns the nearest start that will have at least one other end that
- * together satisfy the constraint
+/* Returns the nearest start that will have at least one other end that
+ * together satisfy the constraint.
  */
 static PedSector
 _constraint_get_nearest_start_soln (const PedConstraint* constraint,
@@ -269,9 +290,9 @@ _constraint_get_nearest_start_soln (const PedConstraint* constraint,
 	return result;
 }
 
-/* given a constraint, and a start ("half of the solution"), find the
+/* Given a constraint, and a start ("half of the solution"), find the
  * range of all possible ends, such that all (start, end) are solutions
- * to constraint (subject to additional alignment requirements)
+ * to constraint (subject to additional alignment requirements).
  */
 static PedGeometry*
 _constraint_get_end_range (const PedConstraint* constraint, PedSector start)
@@ -281,10 +302,9 @@ _constraint_get_end_range (const PedConstraint* constraint, PedSector start)
 	PedSector	last_min_max_end;
 	PedGeometry	end_min_max_range;
 
-	if (start + constraint->min_size > dev->length - 1)
+	if (start + constraint->min_size - 1 > dev->length - 1)
 		return NULL;
 
-
 	first_min_max_end = start + constraint->min_size - 1;
 	last_min_max_end = start + constraint->max_size - 1;
 	if (last_min_max_end > dev->length - 1)
@@ -298,7 +318,7 @@ _constraint_get_end_range (const PedConstraint* constraint, PedSector start)
 		       		       constraint->end_range);
 }
 
-/* given "constraint", and "start", find the end that is nearest to
+/* Given "constraint", and "start", find the end that is nearest to
  * "end", such that ("start", the end) together form a solution to
  * "constraint".
  */
@@ -319,6 +339,10 @@ _constraint_get_nearest_end_soln (const PedConstraint* constraint,
 	return result;
 }
 
+/* Returns the nearest region to "geom" that satisfy a constraint.
+ * Note that "nearest" is somewhat ambiguous.  This function makes
+ * no guarantees about how this ambiguity is resovled.
+ */
 PedGeometry*
 ped_constraint_solve_nearest (
 	const PedConstraint* constraint, const PedGeometry* geom)
@@ -337,7 +361,8 @@ ped_constraint_solve_nearest (
 	if (start == -1)
 		return NULL;
 	end = _constraint_get_nearest_end_soln (constraint, start, geom->end);
-	PED_ASSERT (end != -1, return NULL);
+	if (end == -1)
+		return NULL;
 
 	result = ped_geometry_new (geom->dev, start, end - start + 1);
 	if (!result)
@@ -347,6 +372,10 @@ ped_constraint_solve_nearest (
 	return result;
 }
 
+/* This function finds the largest region that satisfies a constraint.
+ * There might be more than one solution.  This function makes no
+ * guarantees about which solution it will choose in this case.
+ */
 PedGeometry*
 ped_constraint_solve_max (const PedConstraint* constraint)
 {
@@ -360,6 +389,7 @@ ped_constraint_solve_max (const PedConstraint* constraint)
 	return ped_constraint_solve_nearest (constraint, &full_dev);
 }
 
+/* Returns 1 if geom satisfies the given constraint. */
 int
 ped_constraint_is_solution (const PedConstraint* constraint,
 	       		    const PedGeometry* geom)
@@ -384,6 +414,7 @@ ped_constraint_is_solution (const PedConstraint* constraint,
 	return 1;
 }
 
+/* Returns a constraint that any region on the given device will satisfy. */
 PedConstraint*
 ped_constraint_any (const PedDevice* dev)
 {
@@ -401,6 +432,7 @@ ped_constraint_any (const PedDevice* dev)
 			dev->length);
 }
 
+/* Returns a constraint that only the given region will satisfy. */
 PedConstraint*
 ped_constraint_exact (const PedGeometry* geom)
 {
