@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
 #include "config.h"
@@ -30,7 +30,8 @@
 #ifdef DEBUG
 
 int
-ped_assert (int cond, char* cond_text, char* file, int line, char* function)
+ped_assert (int cond, const char* cond_text,
+	    const char* file, int line, const char* function)
 {
 	PedExceptionOption	opt;
 
