@@ -1,6 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
+    Copyright (C) 1999 - 2001, 2005 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
 #define _GNU_SOURCE 1
@@ -180,7 +180,11 @@ ped_device_is_busy (PedDevice* dev)
 {
 	return ped_architecture->dev_ops->is_busy (dev);
 }
-	
+
+/* The meaning of "open" is architecture-dependent.  Apart from requesting
+ * access to the device from the operating system, it does things like flushing
+ * caches.
+ */
 int
 ped_device_open (PedDevice* dev)
 {
@@ -211,6 +215,14 @@ ped_device_close (PedDevice* dev)
 		return ped_architecture->dev_ops->close (dev);
 }
 
+/* This function closes a device, while pretending it is still open.  This is
+ * useful for temporarily suspending libparted access to the device in order
+ * for an external program to access it.  (Running external programs while the
+ * device is open can cause cache coherency problems.)
+ *
+ * In particular, this function keeps track of dev->open_count, so that
+ * reference counting isn't screwed up.
+ */
 int
 ped_device_begin_external_access (PedDevice* dev)
 {
@@ -246,7 +258,7 @@ ped_device_read (PedDevice* dev, void* buffer, PedSector start,
 	PED_ASSERT (!dev->external_mode, return 0);
 	PED_ASSERT (dev->open_count > 0, return 0);
 
-	return ped_architecture->dev_ops->read (dev, buffer, start, count);
+	return (ped_architecture->dev_ops->read) (dev, buffer, start, count);
 }
 
 int
@@ -258,7 +270,7 @@ ped_device_write (PedDevice* dev, const void* buffer, PedSector start,
 	PED_ASSERT (!dev->external_mode, return 0);
 	PED_ASSERT (dev->open_count > 0, return 0);
 
-	return ped_architecture->dev_ops->write (dev, buffer, start, count);
+	return (ped_architecture->dev_ops->write) (dev, buffer, start, count);
 }
 
 PedSector
@@ -269,9 +281,13 @@ ped_device_check (PedDevice* dev, void* buffer, PedSector start,
 	PED_ASSERT (!dev->external_mode, return 0);
 	PED_ASSERT (dev->open_count > 0, return 0);
 
-	return ped_architecture->dev_ops->check (dev, buffer, start, count);
+	return (ped_architecture->dev_ops->check) (dev, buffer, start, count);
 }
 
+/* This function flushes all write-behind caches that might be holding up
+ * writes.  It is slow, because it guarantees cache coherency among all
+ * relevant caches.
+ */
 int
 ped_device_sync (PedDevice* dev)
 {
@@ -282,3 +298,17 @@ ped_device_sync (PedDevice* dev)
 	return ped_architecture->dev_ops->sync (dev);
 }
 
+/* This function flushes all write-behind caches that might be holding writes.
+ * It does NOT ensure cache coherency with other caches that cache data.  If
+ * you need cache coherency, use ped_device_sync() instead.
+ */
+int
+ped_device_sync_fast (PedDevice* dev)
+{
+	PED_ASSERT (dev != NULL, return 0);
+	PED_ASSERT (!dev->external_mode, return 0);
+	PED_ASSERT (dev->open_count > 0, return 0);
+
+	return ped_architecture->dev_ops->sync_fast (dev);
+}
+
