@@ -16,7 +16,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 
     Contributor:  Sven Luther <luther@debian.org>
 */
@@ -273,7 +273,7 @@ struct PartitionBlock {
     uint32_t	pb_Flags;		/* Part Flags (NOMOUNT and BOOTABLE) */
     uint32_t	pb_Reserved1[2];
     uint32_t	pb_DevFlags;		/* Preferred flags for OpenDevice */
-    uint8_t	pb_DriveName[32];	/* Preferred DOS device name: BSTR form */
+    char	pb_DriveName[32];	/* Preferred DOS device name: BSTR form */
     uint32_t	pb_Reserved2[15];
     uint32_t	de_TableSize;		/* Size of Environment vector */
     uint32_t	de_SizeBlock;		/* Size of the blocks in 32 bit words, usually 128 */
@@ -291,7 +291,7 @@ struct PartitionBlock {
     uint32_t	de_MaxTransfer;		/* Max number of bytes to transfer at a time */
     uint32_t	de_Mask;		/* Address Mask to block out certain memory */
     int32_t	de_BootPri;		/* Boot priority for autoboot */
-    uint32_t	de_DosType;		/* Dostype of the filesystem */
+    uint32_t	de_DosType;		/* Dostype of the file system */
     uint32_t	de_Baud;		/* Baud rate for serial handler */
     uint32_t	de_Control;		/* Control word for handler/filesystem */
     uint32_t	de_BootBlocks;		/* Number of blocks containing boot code */
@@ -564,7 +564,7 @@ amiga_read (PedDisk* disk)
 
 		part->num = i;
 		part->type = 0;
-		/* Let's probe what filesystem is present on the disk */
+		/* Let's probe what file system is present on the disk */
 		part->fs_type = ped_file_system_probe (&part->geom);
 		
 		constraint_exact = ped_constraint_exact (&part->geom);
@@ -699,7 +699,7 @@ amiga_write (PedDisk* disk)
 	{
 		ped_exception_throw(PED_EXCEPTION_ERROR,
 			PED_EXCEPTION_CANCEL,
-			_("%s : Failed to list badblocks."), __func__);
+			_("%s : Failed to list bad blocks."), __func__);
 		goto error_free_table;
 	}
 	if (_amiga_find_free_blocks(disk, table, block,
@@ -707,7 +707,7 @@ amiga_write (PedDisk* disk)
 	{
 		ped_exception_throw(PED_EXCEPTION_ERROR,
 			PED_EXCEPTION_CANCEL,
-			_("%s : Failed to list partitionblocks."), __func__);
+			_("%s : Failed to list partition blocks."), __func__);
 		goto error_free_table;
 	}
 	if (_amiga_find_free_blocks(disk, table, block,
@@ -715,7 +715,7 @@ amiga_write (PedDisk* disk)
 	{
 		ped_exception_throw(PED_EXCEPTION_ERROR,
 			PED_EXCEPTION_CANCEL,
-			_("%s : Failed to list filesystemblocks."), __func__);
+			_("%s : Failed to list file system blocks."), __func__);
 		goto error_free_table;
 	}
 	if (_amiga_find_free_blocks(disk, table, block,
@@ -723,7 +723,7 @@ amiga_write (PedDisk* disk)
 	{
 		ped_exception_throw(PED_EXCEPTION_ERROR,
 			PED_EXCEPTION_CANCEL,
-			_("%s : Failed to list bootblocks."), __func__);
+			_("%s : Failed to list boot blocks."), __func__);
 		goto error_free_table;
 	}
 
@@ -750,7 +750,7 @@ amiga_write (PedDisk* disk)
 				block_num);
 			goto error_free_table;
 			/* WARNING : If we fail here, we stop everything,
-			 * and the partition table is lost. A Better
+			 * and the partition table is lost. A better
 			 * solution should be found, using the second
 			 * half of the hardblocks to not overwrite the
 			 * old partition table. It becomes problematic
@@ -1077,7 +1077,7 @@ amiga_partition_enumerate (PedPartition* part)
 	/* failed to allocate a number */
 #ifndef DISCOVER_ONLY
 	ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
-		_("%s : Unable to allocate a partition"), __func__);
+		_("Unable to allocate a partition number."));
 #endif
 	return 0;
 }
