@@ -15,7 +15,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 
     Contributor:  Matt Wilson <msw@redhat.com>
 */
@@ -63,10 +63,10 @@ typedef struct _BSDRawLabel		BSDRawLabel;
 struct _BSDRawPartition {		/* the partition table */
 	uint32_t	p_size;		/* number of sectors in partition */
 	uint32_t	p_offset;	/* starting sector */
-	uint32_t	p_fsize;	/* filesystem basic fragment size */
-	uint8_t		p_fstype;	/* filesystem type, see below */
-	uint8_t		p_frag;		/* filesystem fragments per block */
-	uint16_t	p_cpg;		/* filesystem cylinders per group */
+	uint32_t	p_fsize;	/* file system basic fragment size */
+	uint8_t		p_fstype;	/* file system type, see below */
+	uint8_t		p_frag;		/* file system fragments per block */
+	uint16_t	p_cpg;		/* file system cylinders per group */
 } __attribute__((packed));
 
 struct _BSDRawLabel {
@@ -98,7 +98,7 @@ struct _BSDRawLabel {
 	uint32_t	d_magic2;		/* the magic number (again) */
 	uint16_t	d_checksum;		/* xor of data incl. partitions */
 	
-	/* filesystem and partition information: */
+	/* file system and partition information: */
 	uint16_t	d_npartitions;		/* number of partitions in following */
 	uint32_t	d_bbsize;		/* size of boot area at sn0, bytes */
 	uint32_t	d_sbsize;		/* max size of fs superblock, bytes */
@@ -188,12 +188,15 @@ bsd_alloc (PedDevice* dev)
 	label->d_type = PED_CPU_TO_LE16 (BSD_DTYPE_SCSI);
 	label->d_flags = 0;
 	label->d_secsize = PED_CPU_TO_LE16 (PED_SECTOR_SIZE);
-	label->d_nsectors = PED_CPU_TO_LE32 (dev->sectors);
-	label->d_ntracks = PED_CPU_TO_LE32 (dev->heads);
-	label->d_ncylinders = PED_CPU_TO_LE32 (dev->cylinders);
-	label->d_secpercyl  = PED_CPU_TO_LE32 (dev->sectors * dev->heads);
+	label->d_nsectors = PED_CPU_TO_LE32 (dev->bios_geom.sectors);
+	label->d_ntracks = PED_CPU_TO_LE32 (dev->bios_geom.heads);
+	label->d_ncylinders = PED_CPU_TO_LE32 (dev->bios_geom.cylinders);
+	label->d_secpercyl = PED_CPU_TO_LE32 (dev->bios_geom.sectors
+						* dev->bios_geom.heads);
 	label->d_secperunit
-		= PED_CPU_TO_LE32 (dev->sectors * dev->heads * dev->cylinders);
+		= PED_CPU_TO_LE32 (dev->bios_geom.sectors
+				   * dev->bios_geom.heads
+				   * dev->bios_geom.cylinders);
 	
 	label->d_rpm = PED_CPU_TO_LE16 (3600);
 	label->d_interleave = PED_CPU_TO_LE16 (1);;
@@ -410,6 +413,7 @@ bsd_partition_duplicate (const PedPartition* part)
 				      part->geom.end);
 	if (!new_part)
 		return NULL;
+	new_part->num = part->num;
 
 	old_bsd_data = (BSDPartitionData*) part->disk_specific;
 	new_bsd_data = (BSDPartitionData*) new_part->disk_specific;
@@ -518,7 +522,7 @@ bsd_partition_enumerate (PedPartition* part)
 	/* failed to allocate a number */
 #ifndef DISCOVER_ONLY
 	ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
-			     _("Unable to allocate a bsd disklabel slot"));
+			     _("Unable to allocate a bsd disklabel slot."));
 #endif
 	return 0;
 }
