@@ -1,6 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1999, 2000, 2001, 2004 Free Software Foundation, Inc.
+    Copyright (C) 1999, 2000, 2001, 2004, 2005 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,11 +14,12 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
 #include "config.h"
 
+#include <sys/time.h>
 #include <parted/parted.h>
 #include <parted/debug.h>
 #include <parted/endian.h>
@@ -119,9 +120,11 @@ struct _DosRawPartition {
 } __attribute__((packed));
 
 struct _DosRawTable {
-	char			boot_code [446];
+	char			boot_code [440];
+	uint32_t                mbr_signature;	/* really a unique ID */
+	uint16_t                Unknown;
 	DosRawPartition		partitions [4];
-	uint16_t			magic;
+	uint16_t		magic;
 } __attribute__((packed));
 
 /* OrigState is information we want to preserve about the partition for
@@ -459,9 +462,14 @@ probe_filesystem_for_geom (const PedPartition* part, PedCHSGeometry* bios_geom)
  * We can rewrite this in matrix form:
  *
  * [ c h ] [ cyl_sectors  ]  =  [ s - a ]  =  [ a_ ]
- * [ C H ] [ head_sectors ]     [ S - A ]     [ A_ ]
+ * [ C H ] [ head_sectors ]     [ S - A ]     [ A_ ].
  * 
  * (s - a is abbreviated to a_to simplify the notation.)
+ *
+ * This can be abbreviated into augmented matrix form:
+ *
+ * [ c h | a_ ]
+ * [ C H | A_ ].
  * 
  * Solving these equations requires following the row reduction algorithm.  We
  * need to be careful about a few things though:
@@ -480,6 +488,7 @@ probe_partition_for_geom (PedPartition* part, PedCHSGeometry* bios_geom)
 	RawCHS* end_chs;
 	PedSector c, h, s, a, a_;	/* start */
 	PedSector C, H, S, A, A_;	/* end */
+	PedSector dont_overflow, denum;
 	PedSector cyl_size, head_size;
 	PedSector cylinders, heads, sectors;
 
@@ -510,7 +519,12 @@ probe_partition_for_geom (PedPartition* part, PedCHSGeometry* bios_geom)
 		return 0;
 	if (c > C)
 		return 0;
-	if (A > MAX_CHS_CYLINDER * 255 * 63)
+
+	/* If no geometry is feasible, then don't even bother.
+	 * Useful for eliminating assertions for broken partition
+	 * tables generated by Norton Ghost et al.
+	 */
+	if (A > (C+1) * 255 * 63)
 		return 0;
 
 	/* Not enough information.  In theory, we can do better.  Should we? */
@@ -519,46 +533,57 @@ probe_partition_for_geom (PedPartition* part, PedCHSGeometry* bios_geom)
 	if (C == 0)
 		return 0;
 
-	/* STEP ONE: find cyl_size */
-	if (h == 0 && c > 0) {
-		/* If the matrix looks like this, then it's easy!
-		 * [ c 0 ]
-		 * [ C H ]
-		 */
-		cyl_size = a_ / c;
-	} else if (h > 0) {
-		/* if h > 0, we can do R2 -> R2 - H/h R1 to get:
-		 *
-		 * [ c h ] = [ a_ ]
-		 * [ X 0 ] = [ Y ]
-		 *
-		 * Then, cyl_size = Y / X
-		 *
-		 * However, if H/h isn't an integer, this isn't going
-		 * to work out nicely, so the partition table must
-		 * be inconsistent.
-		 */
-		if (H % h != 0)
-			return 0;
-		if (C == H/h * c)
-			return 0;
-		cyl_size = (A_ - H/h * a_) / (C - H/h * c);
-	} else {
-		/* h == 0 && c == 0: not enough information */
+	/* Calculate the maximum number that can be multiplied by
+	 * any head count without overflowing a PedSector
+	 * 2^8 = 256, 8 bits + 1(sign bit) = 9
+	 */
+	dont_overflow = 1;
+	dont_overflow <<= (8*sizeof(dont_overflow)) - 9;
+	dont_overflow--;
+
+	if (a_ > dont_overflow || A_ > dont_overflow)
+		return 0;
+
+	/* The matrix is solved by :
+	 *
+	 * [ c h | a_]			R1
+	 * [ C H | A_]			R2
+	 *
+	 * (cH - Ch) cyl_size = a_H - A_h		H R1 - h R2
+	 * => (if cH - Ch != 0) cyl_size = (a_H - A_h) / (cH - Ch)
+	 *
+	 * (Hc - hC) head_size = A_c - a_C		c R2 - C R1
+	 * => (if cH - Ch != 0) head_size = (A_c - a_C) / (cH - Ch)
+	 *
+	 *   But this calculation of head_size would need
+	 *   not overflowing A_c or a_C
+	 *   So substitution is use instead, to minimize dimension
+	 *   of temporary results :
+	 *
+	 * If h != 0 : head_size = ( a_ - c cyl_size ) / h
+	 * If H != 0 : head_size = ( A_ - C cyl_size ) / H
+	 *
+	 */
+	denum = c * H - C * h;
+	if (denum == 0)
+		return 0;
+
+	cyl_size = (a_*H - A_*h) / denum;
+	/* Check for non integer result */
+	if (cyl_size * denum != a_*H - A_*h)
 		return 0;
-	}
 
 	PED_ASSERT (cyl_size > 0, return 0);
  	PED_ASSERT (cyl_size <= 255 * 63, return 0);
 
-	/* STEP TWO: find head_size */
-	if (h > 0) {
-		head_size = (a_ - c * cyl_size) / h;
-	} else if (H > 0) {
-		head_size = (A_ - C * cyl_size) / H;
-	} else {
-		/* not enough information */
-		return 0;
+	if (h > 0)
+		head_size = ( a_ - c * cyl_size ) / h;
+	else if (H > 0)
+		head_size = ( A_ - C * cyl_size ) / H;
+	else { 
+		/* should not happen because denum != 0 */
+		head_size = 0;
+		PED_ASSERT (0, return 0);
 	}
 
 	PED_ASSERT (head_size > 0, return 0);
@@ -574,6 +599,13 @@ probe_partition_for_geom (PedPartition* part, PedCHSGeometry* bios_geom)
 	PED_ASSERT (sectors > 0, return 0);
 	PED_ASSERT (sectors <= 63, return 0);
 
+	/* Some broken OEM partitioning program(s) seem to have an out-by-one
+	 * error on the end of partitions.  We should offer to fix the
+	 * partition table...
+	 */
+	if (((C + 1) * heads + H) * sectors + S == A)
+		C++;
+
 	PED_ASSERT ((c * heads + h) * sectors + s == a, return 0);
 	PED_ASSERT ((C * heads + H) * sectors + S == A, return 0);
 
@@ -770,7 +802,8 @@ read_table (PedDisk* disk, PedSector sector, int is_extended_table)
 	if (PED_LE16_TO_CPU (table.magic) != MSDOS_MAGIC) {
 		if (ped_exception_throw (
 			PED_EXCEPTION_ERROR, PED_EXCEPTION_IGNORE_CANCEL,
-			_("Invalid partition table on %s - wrong signature %x"),
+			_("Invalid partition table on %s "
+			  "-- wrong signature %x."),
 			disk->dev->path,
 			PED_LE16_TO_CPU (table.magic))
 				!= PED_EXCEPTION_IGNORE)
@@ -1025,6 +1058,16 @@ write_extended_partitions (PedDisk* disk)
 		return write_empty_table (disk, ext_part->geom.start);
 }
 
+static inline uint32_t generate_random_id()
+{
+	struct timeval tv;
+	int rc;
+	rc = gettimeofday(&tv, NULL);
+	if (rc == -1)
+		return 0;
+	return (uint32_t)(tv.tv_usec & 0xFFFFFFFFUL);
+}
+
 static int
 msdos_write (PedDisk* disk)
 {
@@ -1042,6 +1085,10 @@ msdos_write (PedDisk* disk)
 		memcpy (table.boot_code, MBR_BOOT_CODE, sizeof (MBR_BOOT_CODE));
 	}
 
+	/* If there is no unique identifier, generate a random one */
+	if (!table.mbr_signature)
+		table.mbr_signature = generate_random_id();
+
 	memset (table.partitions, 0, sizeof (DosRawPartition) * 4);
 	table.magic = PED_CPU_TO_LE16 (MSDOS_MAGIC);
 
@@ -1146,8 +1193,13 @@ msdos_partition_destroy (PedPartition* part)
 {
 	PED_ASSERT (part != NULL, return);
 
-	if (ped_partition_is_active (part))
+	if (ped_partition_is_active (part)) {
+		DosPartitionData* dos_data;
+		dos_data = (DosPartitionData*) part->disk_specific;
+		if (dos_data->orig)
+			ped_free (dos_data->orig);
 		ped_free (part->disk_specific);
+	}
 	ped_free (part);
 }
 
@@ -1238,8 +1290,8 @@ msdos_partition_set_flag (PedPartition* part, PedPartitionFlag flag, int state)
 			ped_exception_throw (
 				PED_EXCEPTION_ERROR,
 				PED_EXCEPTION_CANCEL,
-				"Extended partitions can not be hidden on "
-				"msdos disk labels.");
+				_("Extended partitions cannot be hidden on "
+				  "msdos disk labels."));
 			return 0;
 		}
 		dos_data->hidden = state;
@@ -1572,6 +1624,8 @@ _get_min_extended_part_geom (const PedPartition* ext_part,
 		return NULL;
 
 	min_geom = ped_geometry_duplicate (&walk->geom);
+	if (!min_geom)
+		return NULL;
 	ped_geometry_set_start (min_geom, walk->geom.start - 1 * head_size);
 
 	for (walk = ext_part->part_list; walk; walk = walk->next) {
@@ -1641,6 +1695,60 @@ _logical_min_start_head (PedPartition* part, const PedCHSGeometry* bios_geom,
 		return base_head + 1;
 }
 
+/* Shamelessly copied and adapted from _partition_get_overlap_constraint
+ * (in disk.c)
+ * This should get ride of the infamous Assertion (metadata_length > 0) failed
+ * bug for extended msdos disklabels generated by Parted.
+ * 1) There always is a partition table at the start of ext_part, so we leave
+ *    a one sector gap there.
+ * 2)*The partition table of part5 is always at the beginning of the ext_part
+ *    so there is no need to leave a one sector gap before part5.
+ *   *There always is a partition table at the beginning of each partition != 5.
+ * We don't need to worry to much about consistency with 
+ * _partition_get_overlap_constraint because missing it means we are in edge
+ * cases anyway, and we don't lose anything by just refusing to do the job in
+ * those cases.
+ */
+static PedConstraint*
+_log_meta_overlap_constraint (PedPartition* part, PedGeometry* geom)
+{
+	PedGeometry	safe_space;
+	PedSector	min_start;
+	PedSector	max_end;
+	PedPartition*	ext_part = ped_disk_extended_partition (part->disk);
+	PedPartition*	walk;
+	int		not_5 = (part->num != 5);
+
+	PED_ASSERT (ext_part != NULL, return NULL);
+
+	walk = ext_part->part_list;
+
+	/*                                 1)  2)     */
+	min_start = ext_part->geom.start + 1 + not_5;
+	max_end = ext_part->geom.end;
+
+	while (walk != NULL             /*      2)                         2) */
+		&& (   walk->geom.start - (walk->num != 5) < geom->start - not_5
+		    || walk->geom.start - (walk->num != 5) <= min_start )) {
+		if (walk != part && ped_partition_is_active (walk))
+			min_start = walk->geom.end + 1 + not_5; /* 2) */
+		walk = walk->next;
+	}
+
+	while (walk && (walk == part || !ped_partition_is_active (walk)))
+		walk = walk->next;
+
+	if (walk)
+		max_end = walk->geom.start - 1 - (walk->num != 5); /* 2) */
+
+	if (min_start >= max_end)
+		return NULL;
+
+	ped_geometry_init (&safe_space, part->disk->dev,
+			   min_start, max_end - min_start + 1);
+	return ped_constraint_new_from_max (&safe_space);
+}
+
 static int
 _align_logical (PedPartition* part, const PedCHSGeometry* bios_geom,
 		const PedConstraint* constraint)
@@ -1652,9 +1760,16 @@ _align_logical (PedPartition* part, const PedCHSGeometry* bios_geom,
 	PedSector	start_base;
 	int		head;
 	PedGeometry*	solution = NULL;
+	PedConstraint   *intersect, *log_meta_overlap;
 
 	PED_ASSERT (ext_part != NULL, return 0);
 
+	log_meta_overlap = _log_meta_overlap_constraint(part, &part->geom);
+	intersect = ped_constraint_intersect (constraint, log_meta_overlap);
+	ped_constraint_destroy (log_meta_overlap);
+	if (!intersect)
+		return 0;
+
 	start_base = ped_round_down_to (part->geom.start, cyl_size);
 
 	for (head = _logical_min_start_head (part, bios_geom, ext_part, 0);
@@ -1671,10 +1786,12 @@ _align_logical (PedPartition* part, const PedCHSGeometry* bios_geom,
 				_logical_constraint (disk, bios_geom, start, 0);
 
 		solution = _best_solution (part, bios_geom, solution,
-				_try_constraint (part, constraint,
+				_try_constraint (part, intersect,
 						 disk_constraint));
 	}
 
+	ped_constraint_destroy (intersect);
+
 	if (solution) {
 		ped_geometry_set (&part->geom, solution->start,
 				  solution->length);
@@ -1751,39 +1868,13 @@ _align_primary_no_geom (PedPartition* part, const PedConstraint* constraint)
 	return 0;
 }
 
-static PedSector
-_calc_min_logical_start (PedPartition* part)
-{
-	PedDisk*	disk = part->disk;
-	PedPartition*	ext_part = ped_disk_extended_partition (disk);
-	PedPartition*	walk;
-
-	for (walk = ext_part->part_list; walk && walk != part;
-	     walk = walk->next) {
-		if (walk->geom.end > part->geom.start) {
-			if (walk->prev)
-				return walk->prev->geom.end + 2;
-			break;
-		}
-	}
-
-	if (part->num == 5)
-		return ext_part->geom.start + 1;
-	else
-		return ext_part->geom.start + 2;
-}
-
 static int
 _align_logical_no_geom (PedPartition* part, const PedConstraint* constraint)
 {
-	PedDisk*	disk = part->disk;
-	PedPartition*	ext_part = ped_disk_extended_partition (disk);
-	PedSector	min_start = _calc_min_logical_start (part);
 	PedGeometry*	solution;
 
 	solution = _try_constraint (part, constraint,
-			_no_geom_constraint (disk, min_start,
-					     ext_part->geom.end));
+			_log_meta_overlap_constraint (part, &part->geom));
 
 	if (solution) {
 		ped_geometry_set (&part->geom, solution->start,
