@@ -1,6 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 2001, 2002 Free Software Foundation, Inc.
+    Copyright (C) 2001, 2002, 2005 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
 #include <string.h>
@@ -106,7 +106,7 @@ dvh_alloc (PedDevice* dev)
 	dvh_disk_data->boot = 0;
 
 	volume_part = ped_partition_new (disk, PED_PARTITION_EXTENDED, NULL,
-					 1, PTYPE_VOLHDR_DFLTSZ - 1);
+					 0, PTYPE_VOLHDR_DFLTSZ - 1);
 	if (!volume_part)
 		goto error_free_disk_specific;
 	volume_part->num = PNUM_VOLHDR + 1;
@@ -181,56 +181,38 @@ _handle_no_volume_header (PedDisk* disk)
 {
 	PedExceptionOption	ret;
 	PedPartition*		part;
-	PedPartition*		first_free_space;
-	PedConstraint*		constraint_exact;
-
-	for (first_free_space = disk->part_list; first_free_space;
-	     first_free_space = first_free_space->next) {
-		if (first_free_space->type == PED_PARTITION_FREESPACE
-		    && first_free_space->geom.length > 5)
-			break;
-	}
-	/* weird!  Can this happen? */
-	PED_ASSERT (first_free_space != NULL, goto error);
+	PedConstraint*		constraint;
 
 	switch (ped_exception_throw (
 		PED_EXCEPTION_WARNING,
-		PED_EXCEPTION_FIX + PED_EXCEPTION_IGNORE_CANCEL,
-		_("%s has no extended partition (volume header "
-		  "partition).  If you ignore, then any boot volumes "
-		  "will be deleted."),
+		PED_EXCEPTION_FIX + PED_EXCEPTION_CANCEL,
+		_("%s has no extended partition (volume header partition)."),
 		disk->dev->path)) {
-	case PED_EXCEPTION_UNHANDLED:
-	case PED_EXCEPTION_FIX:
-	default:
-		part = ped_partition_new (
-			disk, PED_PARTITION_EXTENDED,
-			NULL,
-			first_free_space->geom.start,
-			first_free_space->geom.start + PTYPE_VOLHDR_DFLTSZ - 1);
-		if (!part)
+		case PED_EXCEPTION_UNHANDLED:
+		case PED_EXCEPTION_FIX:
+		default:
+			part = ped_partition_new (
+				disk, PED_PARTITION_EXTENDED, NULL,
+				0, PTYPE_VOLHDR_DFLTSZ - 1);
+			if (!part)
+				goto error;
+			part->num = PNUM_VOLHDR + 1;
+			constraint = ped_constraint_any (part->disk->dev);
+			if (!constraint)
+				goto error_destroy_part;
+			if (!ped_disk_add_partition (disk, part, constraint))
+				goto error_destroy_constraint;
+			ped_constraint_destroy (constraint);
+			ret = PED_EXCEPTION_FIX;
+			break;
+
+		case PED_EXCEPTION_CANCEL:
 			goto error;
-		part->num = PNUM_VOLHDR + 1;
-		constraint_exact = ped_constraint_exact (&part->geom);
-		if (!constraint_exact)
-			goto error_destroy_part;
-		if (!ped_disk_add_partition (disk, part, constraint_exact))
-			goto error_destroy_constraint_exact;
-		ped_constraint_destroy (constraint_exact);
-		ret = PED_EXCEPTION_FIX;
-		break;
-
-	case PED_EXCEPTION_IGNORE:
-		ret = PED_EXCEPTION_IGNORE;
-		break;
-
-	case PED_EXCEPTION_CANCEL:
-		goto error;
 	}
 	return ret;
 
-error_destroy_constraint_exact:
-	ped_constraint_destroy (constraint_exact);
+error_destroy_constraint:
+	ped_constraint_destroy (constraint);
 error_destroy_part:
 	ped_partition_destroy (part);
 error:
@@ -245,12 +227,6 @@ _parse_partition (PedDisk* disk, struct partition_table* pt)
 	PedSector	start = PED_BE32_TO_CPU (pt->pt_firstlbn);
 	PedSector	length = PED_BE32_TO_CPU (pt->pt_nblks);
 
-	/* the boot/extended's start is often set to 0 */
-	if (start == 0) {
-		start++;
-		length--;
-	}
-
 	part = ped_partition_new (disk,
 			          pt->pt_type ? 0 : PED_PARTITION_EXTENDED,
 				  NULL,
@@ -437,17 +413,8 @@ _generate_partition (PedPartition* part, struct partition_table* pt)
 	/* Assert not a bootfile */
 	PED_ASSERT ((part->type & PED_PARTITION_LOGICAL) == 0, return);
 
-	if (part->geom.start == 1) {
-		/* before, we adjusted the volume header so as not to
-		 * overlap with metadata.  Now, we adjust back...
-		 */
-		pt->pt_nblks = PED_CPU_TO_BE32 (part->geom.length + 1);
-		pt->pt_firstlbn = 0;
-	} else {
-		pt->pt_nblks = PED_CPU_TO_BE32 (part->geom.length);
-		pt->pt_firstlbn = PED_CPU_TO_BE32 (part->geom.start);
-	}
-
+	pt->pt_nblks = PED_CPU_TO_BE32 (part->geom.length);
+	pt->pt_firstlbn = PED_CPU_TO_BE32 (part->geom.start);
 	pt->pt_type = PED_CPU_TO_BE32 (dvh_part_data->type);
 }
 
@@ -762,45 +729,25 @@ dvh_partition_get_name (const PedPartition* part)
 	return dvh_part_data->name;
 }
 
+/* The constraint for the volume header partition is different, because it must
+ * contain the first sector of the disk.
+ */
 static PedConstraint*
-_get_strict_primary_constraint (PedDisk* disk)
+_get_extended_constraint (PedDisk* disk)
 {
-	PedDevice*	dev = disk->dev;
-        PedAlignment    start_align;
-        PedAlignment    end_align;
-        PedGeometry     max_geom;
-	PedSector	block = dev->bios_geom.sectors * dev->bios_geom.heads;
-	PedSector	start_lower_bound;
-	PedPartition*	extended_partition = ped_disk_extended_partition (disk);
-
-	if (extended_partition)
-		start_lower_bound = extended_partition->geom.end + 1;
-	else
-		start_lower_bound = 1;
-
-        if (!ped_alignment_init (&start_align, 0, block))
-                return NULL;
-        if (!ped_alignment_init (&end_align, -1, block))
-                return NULL;
-	if (!ped_geometry_init (&max_geom, dev, start_lower_bound,
-			       	disk->dev->length - 1 - start_lower_bound))
+        PedGeometry     min_geom;
+	if (!ped_geometry_init (&min_geom, disk->dev, 0, 1))
 		return NULL;
-
-        return ped_constraint_new (&start_align, &end_align, &max_geom,
-                                   &max_geom, 1, dev->length);
+        return ped_constraint_new_from_min (&min_geom);
 }
 
 static PedConstraint*
-_get_lax_constraint (PedDisk* disk)
+_get_primary_constraint (PedDisk* disk)
 {
-	PedDevice*	dev = disk->dev;
         PedGeometry     max_geom;
-
-	if (!ped_geometry_init (&max_geom, dev, 1, dev->length - 1))
+	if (!ped_geometry_init (&max_geom, disk->dev, 1, disk->dev->length - 1))
 		return NULL;
-
-        return ped_constraint_new (ped_alignment_any, ped_alignment_any,
-				   &max_geom, &max_geom, 1, dev->length);
+        return ped_constraint_new_from_max (&max_geom);
 }
 
 static int
@@ -808,14 +755,11 @@ dvh_partition_align (PedPartition* part, const PedConstraint* constraint)
 {
         PED_ASSERT (part != NULL, return 0);
 
-	if (part->type == 0) {
-		if (_ped_partition_attempt_align (
-				part, constraint,
-				_get_strict_primary_constraint (part->disk)))
-			return 1;
-	}
-	if (_ped_partition_attempt_align (part, constraint,
-					  _get_lax_constraint (part->disk)))
+	if (_ped_partition_attempt_align (
+			part, constraint,
+			(part->type == PED_PARTITION_EXTENDED)
+				? _get_extended_constraint (part->disk)
+				: _get_primary_constraint (part->disk)))
 		return 1;
 
 #ifndef DISCOVER_ONLY
@@ -880,10 +824,21 @@ static int
 dvh_alloc_metadata (PedDisk* disk)
 {
 	PedPartition* part;
+	PedPartition* extended_part;
 	PedConstraint* constraint_exact;
+	PedPartitionType metadata_type;
 	PED_ASSERT(disk != NULL, return 0);
 
-	part = ped_partition_new (disk, PED_PARTITION_METADATA, NULL, 0, 0);
+	/* We don't need to "protect" the start of the disk from the volume
+	 * header.
+	 */
+	extended_part = ped_disk_extended_partition (disk);
+	if (extended_part && extended_part->geom.start == 0)
+		metadata_type = PED_PARTITION_METADATA | PED_PARTITION_LOGICAL;
+	else
+		metadata_type = PED_PARTITION_METADATA;
+
+	part = ped_partition_new (disk, metadata_type, NULL, 0, 0);
 	if (!part)
 		goto error;
 
