@@ -22,7 +22,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
 #include "config.h"
@@ -52,6 +52,11 @@
 #define MSDOS_MBR_SIGNATURE 0xaa55
 
 #define GPT_HEADER_SIGNATURE 0x5452415020494645LL
+
+/* NOTE: the document that describes revision 1.00 is labelled "version 1.02",
+ * so some implementors got confused...
+ */
+#define GPT_HEADER_REVISION_V1_02 0x00010200
 #define GPT_HEADER_REVISION_V1_00 0x00010000
 #define GPT_HEADER_REVISION_V0_99 0x00009900
 
@@ -207,6 +212,8 @@ typedef struct _GPTPartitionData {
 	int		raid;
 	int		boot;
 	int		hp_service;
+	int 	        hidden;
+	int 	        msftres;
 } GPTPartitionData;
 
 static PedDiskType gpt_disk_type;
@@ -304,7 +311,7 @@ gpt_probe (PedDevice * dev)
 			_("%s contains GPT signatures, indicating that it has "
 			  "a GPT table.  However, it does not have a valid "
 			  "fake msdos partition table, as it should.  Perhaps "
-			  "it was corrupted - possibly by a program that "
+			  "it was corrupted -- possibly by a program that "
 			  "doesn't understand GPT partition tables.  Or "
 			  "perhaps you deleted the GPT table, and are now "
 			  "using an msdos partition table.  Is this a GPT "
@@ -377,8 +384,8 @@ gpt_alloc (PedDevice * dev)
 	ped_geometry_init (&gpt_disk_data->data_area, dev, data_start,
 			   data_end - data_start + 1);
 	gpt_disk_data->entry_count = GPT_DEFAULT_PARTITION_ENTRIES;
-	uuid_generate ((char*) &gpt_disk_data->uuid);
-	swap_uuid_and_efi_guid((char*)(&gpt_disk_data->uuid));
+	uuid_generate ((unsigned char*) &gpt_disk_data->uuid);
+	swap_uuid_and_efi_guid((unsigned char*)(&gpt_disk_data->uuid));
 	return disk;
 
 error_free_disk:
@@ -457,7 +464,7 @@ _parse_header (PedDisk* disk, GuidPartitionTableHeader_t* gpt)
 	PED_ASSERT (_header_is_valid (gpt), return 0);
 
 #ifndef DISCOVER_ONLY
-	if (PED_CPU_TO_LE32 (gpt->Revision) > GPT_HEADER_REVISION_V1_00
+	if (PED_CPU_TO_LE32 (gpt->Revision) > GPT_HEADER_REVISION_V1_02
 	    || PED_CPU_TO_LE32 (gpt->HeaderSize) != 92) {
 		if (ped_exception_throw (
 			PED_EXCEPTION_WARNING,
@@ -509,8 +516,13 @@ _parse_part_entry (PedDisk* disk, GuidPartitionEntry_t* pte)
 					   (uint16_t) pte->PartitionName[i]);
 	gpt_part_data->name[i] = 0;
 
-	gpt_part_data->lvm = gpt_part_data->raid = gpt_part_data->boot
-		= gpt_part_data->hp_service = 0;
+	gpt_part_data->lvm = gpt_part_data->raid 
+		= gpt_part_data->boot = gpt_part_data->hp_service
+		= gpt_part_data->hidden = gpt_part_data->msftres = 0;
+
+	if(pte->Attributes.RequiredToFunction & 0x1)
+		gpt_part_data->hidden = 1;
+	
 	if (!guid_cmp (gpt_part_data->type, PARTITION_SYSTEM_GUID))
 		gpt_part_data->boot = 1;
 	else if (!guid_cmp (gpt_part_data->type, PARTITION_RAID_GUID))
@@ -519,6 +531,8 @@ _parse_part_entry (PedDisk* disk, GuidPartitionEntry_t* pte)
 		gpt_part_data->lvm = 1;
 	else if (!guid_cmp (gpt_part_data->type, PARTITION_HPSERVICE_GUID))
 		gpt_part_data->hp_service = 1;
+	else if (!guid_cmp (gpt_part_data->type, PARTITION_MSFT_RESERVED_GUID))
+		gpt_part_data->msftres = 1;
 
 	return part;
 }
@@ -612,7 +626,7 @@ gpt_read (PedDisk * disk)
 				PED_EXCEPTION_ERROR,
 				PED_EXCEPTION_OK_CANCEL,
 				_("The primary GPT table is corrupt, but the "
-				  "backup appears ok, so that will be used."))
+				  "backup appears OK, so that will be used."))
 				    == PED_EXCEPTION_CANCEL)
 				goto error;
 		} else {
@@ -621,7 +635,7 @@ gpt_read (PedDisk * disk)
 				PED_EXCEPTION_CANCEL,
 				_("Both the primary and backup GPT tables "
 				  "are corrupt.  Try making a fresh table, "
-				  "and using Parted's rescue feature, to "
+				  "and using Parted's rescue feature to "
 				  "recover partitions."));
 			goto error;
 		}
@@ -688,7 +702,10 @@ _write_pmbr(PedDevice * dev)
 	pmbr.PartitionRecord[0].EndSector   = 0xFF;
 	pmbr.PartitionRecord[0].EndTrack    = 0xFF;
 	pmbr.PartitionRecord[0].StartingLBA = PED_CPU_TO_LE32(1);
-	pmbr.PartitionRecord[0].SizeInLBA   = PED_CPU_TO_LE32(dev->length - 1);
+	if ((dev->length - 1ULL) > 0xFFFFFFFFULL) 
+		pmbr.PartitionRecord[0].SizeInLBA = PED_CPU_TO_LE32(0xFFFFFFFF);
+	else
+		pmbr.PartitionRecord[0].SizeInLBA = PED_CPU_TO_LE32(dev->length - 1UL);
 
 	return ped_device_write(dev, &pmbr, GPT_PMBR_LBA, GPT_PMBR_SECTORS);
 }
@@ -703,7 +720,7 @@ _generate_header (PedDisk* disk, int alternate, uint32_t ptes_crc,
 
 	gpt->Signature = PED_CPU_TO_LE64 (GPT_HEADER_SIGNATURE);
 	gpt->Revision = PED_CPU_TO_LE32 (GPT_HEADER_REVISION_V1_00);
-	gpt->HeaderSize = PED_CPU_TO_LE32 (92); /* per 1.02 spec */
+	gpt->HeaderSize = PED_CPU_TO_LE32 (92); /* per 1.00 spec */
 	gpt->HeaderCRC32 = 0;
 	gpt->Reserved1 = 0;
 
@@ -746,6 +763,9 @@ _partition_generate_part_entry (PedPartition* part, GuidPartitionEntry_t* pte)
 	pte->EndingLBA = PED_CPU_TO_LE64(part->geom.end);
 	memset (&pte->Attributes, 0, sizeof (GuidPartitionEntryAttributes_t));
 
+	if (gpt_part_data->hidden)
+		pte->Attributes.RequiredToFunction = 1;
+
 	for (i = 0; i < 72 / sizeof(efi_char16_t); i++)
 		pte->PartitionName[i]
 			= (efi_char16_t) PED_CPU_TO_LE16(
@@ -827,7 +847,7 @@ add_metadata_part(PedDisk * disk, PedSector start, PedSector length)
 
 	constraint_exact = ped_constraint_exact (&part->geom);
 	if (!ped_disk_add_partition (disk, part, constraint_exact))
-		goto error_destroy_part;
+		goto error_destroy_constraint;
 	ped_constraint_destroy (constraint_exact);
 	return 1;
 
@@ -864,8 +884,10 @@ gpt_partition_new(const PedDisk* disk,
 	gpt_part_data->raid = 0;
 	gpt_part_data->boot = 0;
 	gpt_part_data->hp_service = 0;
-	uuid_generate ((char*) &gpt_part_data->uuid);
-	swap_uuid_and_efi_guid((char*)(&gpt_part_data->uuid));
+	gpt_part_data->hidden = 0;
+	gpt_part_data->msftres = 0;
+	uuid_generate ((unsigned char*) &gpt_part_data->uuid);
+	swap_uuid_and_efi_guid((unsigned char*)(&gpt_part_data->uuid));
 	strcpy (gpt_part_data->name, "");
 	return part;
 
@@ -943,13 +965,12 @@ gpt_partition_set_system (PedPartition* part, const PedFileSystemType* fs_type)
 		gpt_part_data->type = PARTITION_HPSERVICE_GUID;
 		return 1;
 	}
+	if (gpt_part_data->msftres) {
+		gpt_part_data->type = PARTITION_MSFT_RESERVED_GUID;
+		return 1;
+	}
 
 	if (fs_type) {
-		if (strncmp (fs_type->name, "fat", 3) == 0
-		    || strcmp (fs_type->name, "ntfs") == 0) {
-			gpt_part_data->type = PARTITION_MSFT_RESERVED_GUID;
-			return 1;
-		}
 		if (strstr (fs_type->name, "swap")) {
 			gpt_part_data->type = PARTITION_SWAP_GUID;
 			return 1;
@@ -1024,30 +1045,48 @@ gpt_partition_set_flag(PedPartition *part,
 	case PED_PARTITION_BOOT:
 		gpt_part_data->boot = state;
 		if (state)
-			gpt_part_data->raid = gpt_part_data->lvm
-				= gpt_part_data->hp_service = 0;
+			gpt_part_data->raid 
+				= gpt_part_data->lvm
+				= gpt_part_data->hp_service
+				= gpt_part_data->msftres = 0;
 		return gpt_partition_set_system (part, part->fs_type);
 	case PED_PARTITION_RAID:
 		gpt_part_data->raid = state;
 		if (state)
-			gpt_part_data->boot = gpt_part_data->lvm
-				= gpt_part_data->hp_service = 0;
+			gpt_part_data->boot
+				= gpt_part_data->lvm
+				= gpt_part_data->hp_service
+				= gpt_part_data->msftres = 0;
 		return gpt_partition_set_system (part, part->fs_type);
 	case PED_PARTITION_LVM:
 		gpt_part_data->lvm = state;
 		if (state)
-			gpt_part_data->boot = gpt_part_data->raid
-				= gpt_part_data->hp_service = 0;
+			gpt_part_data->boot
+				= gpt_part_data->raid
+				= gpt_part_data->hp_service
+				= gpt_part_data->msftres = 0;
 		return gpt_partition_set_system (part, part->fs_type);
 	case PED_PARTITION_HPSERVICE:
 		gpt_part_data->hp_service = state;
 		if (state)
-			gpt_part_data->boot = gpt_part_data->raid
-				= gpt_part_data->lvm = 0;
+			gpt_part_data->boot
+				= gpt_part_data->raid
+				= gpt_part_data->lvm
+				= gpt_part_data->msftres = 0;
 		return gpt_partition_set_system (part, part->fs_type);
+	case PED_PARTITION_MSFT_RESERVED:
+		gpt_part_data->msftres = state;
+		if (state)
+			gpt_part_data->boot
+				= gpt_part_data->raid
+				= gpt_part_data->lvm
+				= gpt_part_data->hp_service = 0;
+		return gpt_partition_set_system (part, part->fs_type);
+	case PED_PARTITION_HIDDEN:
+		gpt_part_data->hidden = state;
+		return 1;
 	case PED_PARTITION_SWAP:
 	case PED_PARTITION_ROOT:
-	case PED_PARTITION_HIDDEN:
 	case PED_PARTITION_LBA:
 	default:
 		return 0;
@@ -1071,10 +1110,13 @@ gpt_partition_get_flag(const PedPartition *part, PedPartitionFlag flag)
 		return gpt_part_data->boot;
 	case PED_PARTITION_HPSERVICE:
 		return gpt_part_data->hp_service;
+	case PED_PARTITION_MSFT_RESERVED:
+		return gpt_part_data->msftres;
+	case PED_PARTITION_HIDDEN:
+		return gpt_part_data->hidden;
 	case PED_PARTITION_SWAP:
 	case PED_PARTITION_LBA:
 	case PED_PARTITION_ROOT:
-	case PED_PARTITION_HIDDEN:
 	default:
 		return 0;
 	}
@@ -1090,10 +1132,11 @@ gpt_partition_is_flag_available(const PedPartition * part,
 	case PED_PARTITION_LVM:
 	case PED_PARTITION_BOOT:
 	case PED_PARTITION_HPSERVICE:
+	case PED_PARTITION_MSFT_RESERVED:
+	case PED_PARTITION_HIDDEN:
 		return 1;
 	case PED_PARTITION_SWAP:
 	case PED_PARTITION_ROOT:
-	case PED_PARTITION_HIDDEN:
 	case PED_PARTITION_LBA:
 	default:
 		return 0;
