@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
 #include "config.h"
@@ -202,8 +202,12 @@ loop_partition_new (const PedDisk* disk, PedPartitionType part_type,
 static PedPartition*
 loop_partition_duplicate (const PedPartition* part)
 {
-	return ped_partition_new (part->disk, part->type, part->fs_type,
-				  part->geom.start, part->geom.end);
+	PedPartition* result;
+	
+	result = ped_partition_new (part->disk, part->type, part->fs_type,
+				    part->geom.start, part->geom.end);
+	result->num = part->num;
+	return result;
 }
 
 static void
