@@ -1,6 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 2000, 2002 Free Software Foundation, Inc.
+    Copyright (C) 2000, 2002, 2004 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
 #include "config.h"
@@ -46,7 +46,8 @@
  *	Apple_Driver		device driver
  *	Apple_Driver43		SCSI Manager 4.3 device driver
  *	Apple_MFS		original Macintosh File System 
- *	Apple_HFS		Hierarchical File System
+ *	Apple_HFS		Hierarchical File System (and +)
+ *	Apple_HFSX		HFS+ with case sensitivity and more
  *	Apple_UNIX_SVR2		UNIX file system (UFS?)
  *	Apple_PRODOS		ProDOS file system 
  *	Apple_Free		unused space
@@ -127,6 +128,8 @@ typedef struct {
 	int		is_driver;
 	int		is_root;
 	int		is_swap;
+	int		is_lvm;
+	int		is_raid;
 
 	PedSector	data_region_length;
 	PedSector	boot_region_length;
@@ -209,6 +212,8 @@ _disk_add_part_map_entry (PedDisk* disk, int warn)
 
 	part_map_size
 		= ped_round_up_to (mac_disk_data->last_part_entry_num, 64);
+	if (part_map_size == 0)
+		part_map_size = 64;
 
 	new_part = ped_partition_new (disk, 0, NULL, 1, part_map_size - 1);
 	if (!new_part)
@@ -258,7 +263,7 @@ mac_alloc (PedDevice* dev)
 		goto error;
 
 	mac_disk_data = (MacDiskData*) ped_malloc (sizeof (MacDiskData));
-	if (!disk->disk_specific)
+	if (!mac_disk_data)
 		goto error_free_disk;
 	disk->disk_specific = mac_disk_data;
 	mac_disk_data->ghost_size = disk->dev->sector_size / 512;
@@ -282,6 +287,7 @@ mac_duplicate (const PedDisk* disk)
 	PedDisk*	new_disk;
 	MacDiskData*	new_mac_data;
 	MacDiskData*	old_mac_data = (MacDiskData*) disk->disk_specific;
+	PedPartition*	partition_map;
        
 	new_disk = ped_disk_new_fresh (disk->dev, &mac_disk_type);
 	if (!new_disk)
@@ -289,6 +295,13 @@ mac_duplicate (const PedDisk* disk)
 
 	new_mac_data = (MacDiskData*) new_disk->disk_specific;
 
+	/* remove the partition map partition - it will be duplicated
+	 * later.
+	 */
+	partition_map = ped_disk_get_partition_by_sector (new_disk, 1);
+	PED_ASSERT (partition_map != NULL, return 0);
+	ped_disk_remove_partition (new_disk, partition_map);
+
 	/* ugly, but C is ugly :p */
 	memcpy (new_mac_data, old_mac_data, sizeof (MacDiskData));
 	return new_disk;
@@ -346,7 +359,13 @@ mac_clobber (PedDevice* dev)
 static int
 _rawpart_cmp_type (MacRawPartition* raw_part, char* type)
 {
-	return strcasecmp (raw_part->type, type) == 0;
+	return strncasecmp (raw_part->type, type, 32) == 0;
+}
+
+static int
+_rawpart_cmp_name (MacRawPartition* raw_part, char* name)
+{
+	return strncasecmp (raw_part->name, name, 32) == 0;
 }
 
 static int
@@ -405,24 +424,42 @@ _rawpart_is_swap (MacRawPartition* raw_part)
 	return 1;
 }
 
+static int
+_rawpart_is_lvm (MacRawPartition* raw_part)
+{
+	if (strcmp (raw_part->type, "Linux_LVM") != 0)
+		return 0;
+	return 1;
+}
+
+static int
+_rawpart_is_raid (MacRawPartition* raw_part)
+{
+	if (strcmp (raw_part->type, "Linux_RAID") != 0)
+		return 0;
+	return 1;
+}
+
 static int
 _rawpart_is_void (MacRawPartition* raw_part)
 {
 	return _rawpart_cmp_type (raw_part, "Apple_Void");
 }
 
-/* returns 1 if the raw_part represents a partition that is "unused space",
- * or doesn't represent a partition at all.  NOTE: some people make
- * Apple_Free partitions with MacOS, because they can't select another
- * type.  So, if the name is anything other than "Extra", it is treated
- * as a "real" partition.
+/* returns 1 if the raw_part represents a partition that is "unused space", or
+ * doesn't represent a partition at all.  NOTE: some people make Apple_Free
+ * partitions with MacOS, because they can't select another type.  So, if the
+ * name is anything other than "Extra" or "", it is treated as a "real"
+ * partition.
  */
 static int
 _rawpart_is_active (MacRawPartition* raw_part)
 {
-	if (_rawpart_cmp_type (raw_part, "Apple_Void"))
+	if (_rawpart_cmp_type (raw_part, "Apple_Free")
+	    && (_rawpart_cmp_name (raw_part, "Extra")
+		|| _rawpart_cmp_name (raw_part, "")))
 		return 0;
-	if (_rawpart_cmp_type (raw_part, "Apple_Free"))
+	if (_rawpart_cmp_type (raw_part, "Apple_Void"))
 		return 0;
 	if (_rawpart_cmp_type (raw_part, "Apple_Scratch"))
 		return 0;
@@ -483,6 +520,8 @@ _rawpart_analyse (MacRawPartition* raw_part, PedDisk* disk, int num)
 	mac_part_data->is_driver = _rawpart_is_driver (raw_part);
 	mac_part_data->is_root = _rawpart_is_root (raw_part);
 	mac_part_data->is_swap = _rawpart_is_swap (raw_part);
+	mac_part_data->is_lvm = _rawpart_is_lvm (raw_part);
+	mac_part_data->is_raid = _rawpart_is_raid (raw_part);
 
 	/* "data" region */
 #ifndef DISCOVER_ONLY
@@ -491,7 +530,7 @@ _rawpart_analyse (MacRawPartition* raw_part, PedDisk* disk, int num)
 			PED_EXCEPTION_ERROR,
 			PED_EXCEPTION_CANCEL,
 			_("The data region doesn't start at the start "
-			  "of the partition"));
+			  "of the partition."));
 		goto error_destroy_part;
 	}
 #endif /* !DISCOVER_ONLY */
@@ -508,7 +547,7 @@ _rawpart_analyse (MacRawPartition* raw_part, PedDisk* disk, int num)
 			PED_EXCEPTION_ERROR,
 			PED_EXCEPTION_CANCEL,
 			_("The boot region doesn't start at the start "
-			  "of the partition"));
+			  "of the partition."));
 		goto error_destroy_part;
 	}
 #endif
@@ -699,8 +738,8 @@ mac_read (PedDisk* disk)
 				PED_EXCEPTION_ERROR,
 				PED_EXCEPTION_IGNORE_CANCEL,
 				_("Conflicting partition map entry sizes!  "
-				  "Entry 1 says it's %d, but entry %d says its "
-				  "%d!"),
+				  "Entry 1 says it is %d, but entry %d says "
+				  "it is %d!"),
 				last_part_entry_num,
 				_rawpart_get_partmap_size (&raw_part, disk))
 					!= PED_EXCEPTION_IGNORE)
@@ -725,7 +764,8 @@ mac_read (PedDisk* disk)
 			    && ped_exception_throw (
 					PED_EXCEPTION_ERROR,
 					PED_EXCEPTION_IGNORE_CANCEL,
-					_("Weird - 2 partitions map entries!"))
+					_("Weird!  There are 2 partitions "
+					  "map entries!"))
 			    != PED_EXCEPTION_IGNORE)
 				goto error_delete_all;
 
@@ -1005,6 +1045,8 @@ mac_partition_new (
 		mac_data->is_boot = 0;
 		mac_data->is_root = 0;
 		mac_data->is_swap = 0;
+		mac_data->is_lvm = 0;
+		mac_data->is_raid = 0;
 
 		strcpy (mac_data->volume_name, "untitled");
 	} else {
@@ -1068,9 +1110,13 @@ mac_partition_set_system (PedPartition* part, const PedFileSystemType* fs_type)
 		return 1;
 	}
 
-	if (fs_type && !strcmp (fs_type->name, "hfs")) {
+	if (fs_type && (!strcmp (fs_type->name, "hfs")
+			|| !strcmp (fs_type->name, "hfs+"))) {
 		strcpy (mac_data->system_name, "Apple_HFS");
 		mac_data->status |= 0x7f;
+	} else if (fs_type && !strcmp (fs_type->name, "hfsx")) {
+		strcpy (mac_data->system_name, "Apple_HFSX");
+		mac_data->status |= 0x7f;
 	} else {
 		strcpy (mac_data->system_name, "Apple_UNIX_SVR2");
 		mac_data->status = 0x33;
@@ -1096,9 +1142,11 @@ mac_partition_set_flag (PedPartition* part, PedPartitionFlag flag, int state)
 
 		if (part->fs_type)
 			return mac_partition_set_system (part, part->fs_type);
-			
-		strcpy (mac_data->system_name, "Apple_Bootstrap");
-		mac_data->status = 0x33;
+
+		if (state) {
+			strcpy (mac_data->system_name, "Apple_Bootstrap");
+			mac_data->status = 0x33;
+		}
 		return 1;
 
 	case PED_PARTITION_ROOT:
@@ -1123,6 +1171,22 @@ mac_partition_set_flag (PedPartition* part, PedPartitionFlag flag, int state)
 		mac_data->is_swap = state;
 		return 1;
 
+	case PED_PARTITION_LVM:
+		mac_data->is_lvm = state;
+		if (state)
+			strcpy (mac_data->system_name, "Linux_LVM");
+		else
+			mac_partition_set_system (part, part->fs_type);
+		return 1;
+
+	case PED_PARTITION_RAID:
+		mac_data->is_raid = state;
+		if (state)
+			strcpy (mac_data->system_name, "Linux_RAID");
+		else
+			mac_partition_set_system (part, part->fs_type);
+		return 1;
+
 	default:
 		return 0;
 	}
@@ -1147,6 +1211,9 @@ mac_partition_get_flag (const PedPartition* part, PedPartitionFlag flag)
 	case PED_PARTITION_SWAP:
 		return mac_data->is_swap;
 
+	case PED_PARTITION_LVM:
+		return mac_data->is_lvm;
+
 	default:
 		return 0;
 	}
@@ -1160,6 +1227,7 @@ mac_partition_is_flag_available (
 	case PED_PARTITION_BOOT:
 	case PED_PARTITION_ROOT:
 	case PED_PARTITION_SWAP:
+	case PED_PARTITION_LVM:
 		return 1;
 
 	default:
@@ -1278,7 +1346,8 @@ mac_partition_enumerate (PedPartition* part)
 	ped_exception_throw (
 		PED_EXCEPTION_ERROR,
 		PED_EXCEPTION_CANCEL,
-		_("Can't add another partition - partition map is too small!"));
+		_("Can't add another partition -- the partition map is too "
+		  "small!"));
 #endif
 
 	return 0;
