@@ -1,5 +1,5 @@
 /* 
-    libparted/fs_amiga - amiga filesystem support.
+    libparted/fs_amiga - amiga file system support.
     Copyright (C) 2000, 2001 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 
     Contributor:  Sven Luther <luther@debian.org>
 */
@@ -347,7 +347,7 @@ amiga_find_part (PedGeometry *geom, struct PartitionBlock *part)
 		if (!ped_device_read (geom->dev, part, (PedSector)partblock, 1)) {
 			switch (ped_exception_throw(PED_EXCEPTION_ERROR,
 				PED_EXCEPTION_CANCEL,
-				_("%s : Failed to to read partition block %llu\n"),
+				_("%s : Failed to read partition block %llu\n"),
 				__func__, (PedSector)partblock))
 			{
 				case PED_EXCEPTION_CANCEL :
