@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #include "config.h"
@@ -88,7 +88,7 @@ blk_t ext2_find_free_block(struct ext2_fs *fs)
 		}
 
 	ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
-			     _("Filesystem full!"));
+			     _("File system full!"));
 	return 0;
 }
 
@@ -113,7 +113,7 @@ ino_t ext2_find_free_inode(struct ext2_fs *fs)
 		}
 
 	ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
-			     _("Filesystem full!"));
+			     _("File system full!"));
 	return 0;
 }
 
@@ -511,7 +511,7 @@ int ext2_do_inode(struct ext2_fs *fs, struct ext2_inode *inode, blk_t block,
 
 	/* FIXME: we should check for triple-indirect blocks here, but it
 	 * would be nice to have a better routine to traverse blocks, and
-	 * filesystems that need triple-indirect blocks for the resize
+	 * file systems that need triple-indirect blocks for the resize
 	 * inode are too big to worry about yet.
 	 */
 
@@ -693,7 +693,7 @@ static int ext2_determine_itoffset(struct ext2_fs *fs)
 		{
 			ped_exception_throw (PED_EXCEPTION_NO_FEATURE,
 				PED_EXCEPTION_CANCEL,
-			_("This ext2 filesystem has a rather strange layout!  "
+			_("This ext2 file system has a rather strange layout!  "
 			  "Parted can't resize this (yet)."));
 			return 0;
 		}
@@ -725,7 +725,7 @@ struct ext2_fs *ext2_open(struct ext2_dev_handle *handle, int state)
 	{
 		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
 			_("Invalid superblock.  Are you sure this is an ext2 "
-			  "filesystem?"));
+			  "file system?"));
 		goto error_free_fs;
 	}
 
@@ -738,7 +738,7 @@ struct ext2_fs *ext2_open(struct ext2_dev_handle *handle, int state)
 	{
 		if (ped_exception_throw (
 			PED_EXCEPTION_WARNING, PED_EXCEPTION_IGNORE_CANCEL,
-			_("Filesystem has errors!  You should run e2fsck."))
+			_("File system has errors!  You should run e2fsck."))
 				== PED_EXCEPTION_CANCEL)
 			goto error_free_fs;
 	}
@@ -749,9 +749,9 @@ struct ext2_fs *ext2_open(struct ext2_dev_handle *handle, int state)
 	{
 		if (ped_exception_throw (
 			PED_EXCEPTION_ERROR, PED_EXCEPTION_IGNORE_CANCEL,
-			_("Filesystem was not cleanly unmounted!  "
-			  "You should e2fsck.  Modifying an unclean filesystem "
-			  "could cause severe corruption."))
+			_("File system was not cleanly unmounted!  "
+			  "You should run e2fsck.  Modifying an unclean "
+			  "file system could cause severe corruption."))
 				!= PED_EXCEPTION_IGNORE)
 			goto error_free_fs;
 	}
@@ -769,7 +769,7 @@ struct ext2_fs *ext2_open(struct ext2_dev_handle *handle, int state)
 			    EXT2_FEATURE_RO_COMPAT_LARGE_FILE)))
 	{
 		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
-		     _("Filesystem has incompatible feature enabled"));
+		     _("File system has an incompatible feature enabled."));
 		goto error_free_fs;
 	}
 
