@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #include "config.h"
@@ -26,7 +26,18 @@
 #include "ext2.h"
 
 
-
+/* This struct describes a single block that will be relocated.  The
+ * block's original location is "num", and its new location is "dest".
+ * The block is presumebly referred to by some other block in the file
+ * system, which is recorded as "refblock".  (Only one reference to
+ * the block is allowed by the block relocator.)  "refoffset" describes
+ * the location within the refblock in which the block is referenced.
+ * "isindirect" is 0 for direct, 1 for single-indirect, 2 for
+ * double-indirect, etc.
+ *
+ * The algorithms in the file fill the entries of this struct in this order:
+ * num, refblock/refoffset/isindirectblock, dest.
+ */
 struct ext2_block_entry
 {
 	blk_t		num;
@@ -36,6 +47,35 @@ struct ext2_block_entry
 	unsigned	isindirectblock:16;
 };
 
+/* This struct contains all data structures relevant to the block relocator.
+ * 	- newallocoffset is the distance between the start of a block group,
+ * 	and the first data block in the group.  This can change when a
+ * 	filesystem is resized, because the size of the group descriptors is
+ * 	proportional to the size of the filesystem.
+ * 
+ * 	- allocentries is the size of the "block" array.  It is a tuneable
+ * 	parameter that determines how many blocks can be moved in each
+ * 	pass.
+ * 
+ * 	- usedentries says how many entries of the "block" array have been
+ * 	used.  That is, how many blocks have been scheduled so far to
+ * 	be moved.
+ *
+ * 	- resolvedentries is the number of blocks whose referencing block
+ * 	has been found and recorded in block[.]->refblock, etc.
+ *
+ * 	- block is an array that records which blocks need to be moved, and
+ * 	where they will be moved to, etc.  At some point in the algorithm, this
+ * 	array gets sorted (grep for qsort!) by indirectness.
+ *
+ * 	- start: each entry in this array corresponds to a level of
+ * 	indirectness (0-3).  Each level has two items: dst and num.  "num"
+ * 	is the number of blocks inside "block" of that level of indirectness.
+ * 	After doscan() is finished, and the level of indirectness of each
+ * 	block is known, "block" is sorted (see above).  The "dst" pointer
+ * 	is a pointer inside "block" that indicates the start of the portion
+ * 	of the array containg blocks of that level of indirectness.
+ */
 struct ext2_block_relocator_state
 {
 	blk_t			 newallocoffset;
@@ -137,6 +177,9 @@ struct ext2_block_entry *findit(struct ext2_block_relocator_state *state, blk_t
 	return retv;
 }
 
+/* This function adds records a reference to a block ("blk"), if that
+ * block is scheduled to be moved.
+ */
 static int doblock(struct ext2_fs *fs,
 		   struct ext2_block_relocator_state *state,
 		   blk_t blk,
@@ -152,7 +195,7 @@ static int doblock(struct ext2_fs *fs,
 	if (ent->refblock)
 	{
 		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
-			_("Cross-linked blocks found! better go run e2fsck "
+			_("Cross-linked blocks found!  Better go run e2fsck "
 			  "first!"));
 		return 0;
 	}
@@ -258,7 +301,9 @@ static int dotindblock(struct ext2_fs *fs,
 }
 
 
-
+/* This function records any block references from an inode to blocks that are
+ * scheduled to be moved.
+ */
 static int doinode(struct ext2_fs *fs, struct ext2_block_relocator_state *state, int inode)
 {
 	struct ext2_inode buf;
@@ -327,6 +372,9 @@ static int doinode(struct ext2_fs *fs, struct ext2_block_relocator_state *state,
 	return 1;
 }
 
+/* This function scans the entire filesystem, to find all references to blocks
+ * that are scheduled to be moved.
+ */
 static int doscan(struct ext2_fs *fs, struct ext2_block_relocator_state *state)
 {
 	int i;
@@ -486,7 +534,7 @@ static int ext2_block_relocator_ref(struct ext2_fs *fs, struct ext2_block_reloca
 	if (!(block->refblock || block->refoffset))
 	{
 		ped_exception_throw (PED_EXCEPTION_BUG, PED_EXCEPTION_CANCEL,
-				     _("Block %i has no reference?  Weird"),
+				     _("Block %i has no reference?  Weird."),
 				     block->num);
 		return 0;
 	}
@@ -540,6 +588,14 @@ static int ext2_block_relocator_ref(struct ext2_fs *fs, struct ext2_block_reloca
 	return 1;
 }
 
+/* This function allocates new locations for blocks that are scheduled to move
+ * (inside state->blocks).
+ *
+ * FIXME: doesn't seem to handle sparse block groups.  That is, there might be
+ * some free space that could be exploited in resizing that currently isn't...
+ *
+ * FIXME: should throw an exception if it fails to allocate blocks.
+ */
 static int ext2_block_relocator_grab_blocks(struct ext2_fs *fs, struct ext2_block_relocator_state *state)
 {
 	int i;
