@@ -14,13 +14,13 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
-static const char _ext2_buffer_c[] = "$Id: ext2_buffer.c,v 1.5 1999/10/02 20:56:47 aclausen Exp $";
-
 #include "config.h"
 
+#ifndef DISCOVER_ONLY
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -442,3 +442,6 @@ int ext2_brelse(struct ext2_buffer_head *bh, int forget)
 	}
 	return 1;
 }
+
+#endif /* !DISCOVER_ONLY */
+
