@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #include "config.h"
@@ -112,7 +112,7 @@ static int addref(struct ext2_fs *fs, struct ext2_inode_relocator_state *state,
 	{
 		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
 			_("Found an inode with a incorrect link count.  "
-			  "Better go run e2fsck first."));
+			  "Better go run e2fsck first!"));
 		return 0;
 	}
 
