@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #include "config.h"
@@ -87,7 +87,7 @@ static int ext2_mkfs_write_main(struct ext2_dev_handle *handle,
 	if (EXT2_SUPER_LOG_BLOCK_SIZE(*sb))
 	{
 		sbbuf = ped_malloc(1024 << EXT2_SUPER_LOG_BLOCK_SIZE(*sb));
-		if (!handle->ops->read(handle->cookie, sbbuf, 0, 1))
+		if (!(handle->ops->read)(handle->cookie, sbbuf, 0, 1))
 			return 0;
 		memcpy(sbbuf+1024, sb, 1024);
 		freeit = 1;
