@@ -14,14 +14,12 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 */
 
 #ifndef _PARTED_IO_H
 #define _PARTED_IO_H
 
-static const char _parted_io_h[] = "$Id: $";
-
 #include "ext2.h"
 
 void ext2_destroy_dev_handle(struct ext2_dev_handle *handle);
