@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
 #include <string.h>
@@ -291,6 +291,8 @@ fat_create (PedGeometry* geom, FatType fat_type, PedTimer* timer)
 	fs_info = (FatSpecific*) fs->type_specific;
 
 	fs_info->logical_sector_size = 1;
+	fs_info->sectors_per_track = geom->dev->bios_geom.sectors;
+	fs_info->heads = geom->dev->bios_geom.heads;
 	fs_info->sector_count = fs->geom->length;
 	fs_info->fat_table_count = 2;
 /* some initial values, to be changed later */
@@ -307,7 +309,7 @@ fat_create (PedGeometry* geom, FatType fat_type, PedTimer* timer)
 			&fs_info->fat_sectors)) {
 		ped_exception_throw (PED_EXCEPTION_ERROR,
 			PED_EXCEPTION_CANCEL,
-			_("Partition too big/small for a %s file system"),
+			_("Partition too big/small for a %s file system."),
 			(fat_type == FAT_TYPE_FAT16)
 		       		? fat16_type.name
 				: fat32_type.name);
@@ -433,7 +435,7 @@ fat_close (PedFileSystem* fs)
 	return 1;
 }
 
-/* Hack: just resize the filesystem outside of it's boundaries! */
+/* Hack: just resize the file system outside of its boundaries! */
 PedFileSystem*
 fat_copy (const PedFileSystem* fs, PedGeometry* geom, PedTimer* timer)
 {
@@ -474,9 +476,9 @@ _compare_fats (PedFileSystem* fs)
 			if (ped_exception_throw (PED_EXCEPTION_ERROR,
 				PED_EXCEPTION_IGNORE_CANCEL,
 				_("The FATs don't match.  If you don't know "
-				"what this means, then select cancel, run "
-				"scandisk on the file system, and then come "
-				"back."))
+				  "what this means, then select cancel, run "
+				  "scandisk on the file system, and then come "
+				  "back."))
 			    != PED_EXCEPTION_IGNORE)
 				goto error_free_table_copy;
 		}
@@ -526,10 +528,10 @@ fat_check (PedFileSystem* fs, PedTimer* timer)
 			if (ped_exception_throw (PED_EXCEPTION_WARNING,
 				PED_EXCEPTION_IGNORE_CANCEL,
 				_("File system doesn't have expected sizes for "
-				"Windows to like it.  "
-				"Cluster size is %dk (%dk expected); "
-				"number of clusters is %d (%d expected); "
-				"size of FATs is %d sectors (%d expected)."),
+				  "Windows to like it.  "
+				  "Cluster size is %dk (%dk expected); "
+				  "number of clusters is %d (%d expected); "
+				  "size of FATs is %d sectors (%d expected)."),
 				(int) fs_info->cluster_sectors / 2,
 					(int) cluster_sectors / 2,
 				(int) fs_info->cluster_count,
@@ -544,11 +546,12 @@ fat_check (PedFileSystem* fs, PedTimer* timer)
 	if (fs_info->fat_type == FAT_TYPE_FAT32) {
 		info_free_clusters
 			= PED_LE32_TO_CPU (fs_info->info_sector.free_clusters);
-		if (info_free_clusters != fs_info->fat->free_cluster_count) {
+		if (info_free_clusters != (FatCluster) -1
+		    && info_free_clusters != fs_info->fat->free_cluster_count) {
 			if (ped_exception_throw (PED_EXCEPTION_WARNING,
 				PED_EXCEPTION_IGNORE_CANCEL,
 				_("File system is reporting the free space as "
-				"%d clusters, not %d clusters."),
+				  "%d clusters, not %d clusters."),
 				info_free_clusters,
 				fs_info->fat->free_cluster_count)
 					!= PED_EXCEPTION_IGNORE)
@@ -868,7 +871,7 @@ ped_file_system_fat_init ()
 {
 	if (sizeof (FatBootSector) != 512) {
 		ped_exception_throw (PED_EXCEPTION_BUG, PED_EXCEPTION_CANCEL,
-			_("GNU parted was miscompiled: the FAT boot sector "
+			_("GNU Parted was miscompiled: the FAT boot sector "
 			"should be 512 bytes.  FAT support will be disabled."));
 	} else {
 		ped_file_system_type_register (&fat16_type);
