@@ -1,6 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1999, 2000 Free Software Foundation, Inc.
+    Copyright (C) 1999, 2000, 2002 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,25 +14,18 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
-/* It's a bit silly calling a swap partition a filesystem.  Oh well...  */
+/* It's a bit silly calling a swap partition a file system.  Oh well...  */
 
 #include "config.h"
 
 #include <parted/parted.h>
 #include <parted/endian.h>
-#include <parted/disk_dos.h>
-#include <parted/disk_loop.h>
-#include <parted/disk_pc98.h>
-#include <parted/disk_mac.h>
-#include <parted/disk_bsd.h>
-#include <parted/disk_gpt.h>
-#include <parted/disk_sun.h>
 
-#include <libintl.h>
 #if ENABLE_NLS
+#  include <libintl.h>
 #  define _(String) dgettext (PACKAGE, String)
 #else
 #  define _(String) (String)
@@ -63,7 +56,6 @@ typedef struct {
 		SwapNewHeader	new;
 		SwapOldHeader	old;
 	}* header;
-	char*		sig;		/* will point inside header */
 
 	void*		buffer;
 	int		buffer_size;
@@ -74,72 +66,50 @@ typedef struct {
 	unsigned int	max_bad_pages;
 } SwapSpecific;
 
-static int swap_probe (const PedGeometry* geom);
-static int swap_clobber (PedGeometry* geom);
-static PedFileSystem* swap_open (const PedGeometry* geom);
-static PedFileSystem* swap_create (PedGeometry* geom);
+static PedFileSystemType swap_type;
+
+static PedFileSystem* swap_open (PedGeometry* geom);
 static int swap_close (PedFileSystem* fs);
-static int swap_resize (PedFileSystem* fs, const PedGeometry* geom);
-static int swap_copy (const PedFileSystem* fs, PedGeometry* geom);
-static int swap_check (PedFileSystem* fs);
-static PedConstraint* swap_get_resize_constraint (const PedFileSystem* fs);
-static int swap_set_system (const PedFileSystem* fs, PedPartition* part,
-			    const PedDiskType* disk_type);
 
-static PedFileSystemOps swap_ops = {
-	probe:		swap_probe,
-	clobber:	swap_clobber,
-	open:		swap_open,
-	create:		swap_create,
-	close:		swap_close,
-	check:		swap_check,
-	copy:		swap_copy,
-	resize:		swap_resize,
-	get_resize_constraint:	swap_get_resize_constraint,
-	set_system:	swap_set_system
-};
-
-static PedFileSystemType swap_type = {
-	next:	NULL,
-	ops:	&swap_ops,
-	name:	"linux-swap"
-};
-
-void
-ped_file_system_linux_swap_init ()
-{
-	ped_file_system_type_register (&swap_type);
-}
-
-void
-ped_file_system_linux_swap_done ()
-{
-	ped_file_system_type_unregister (&swap_type);
-}
-
-static int
-swap_probe (const PedGeometry* geom)
+static PedGeometry*
+swap_probe (PedGeometry* geom)
 {
 	PedFileSystem*	fs;
+	SwapSpecific*	fs_info;
+	PedGeometry*	probed_geom;
+	PedSector	length;
+
 	fs = swap_open (geom);
-	if (fs) {
-		swap_close (fs);
-		return 1;
-	}
-	return 0;
+	if (!fs)
+		goto error;
+	fs_info = SWAP_SPECIFIC (fs);
+
+	if (fs_info->version)
+		length = fs_info->page_sectors * fs_info->page_count;
+	else
+		length = geom->length;
+	probed_geom = ped_geometry_new (geom->dev, geom->start, length);
+	if (!probed_geom)
+		goto error_close_fs;
+	swap_close (fs);
+	return probed_geom;
+
+error_close_fs:
+	swap_close (fs);
+error:
+	return NULL;
 }
 
+#ifndef DISCOVER_ONLY
 static int
 swap_clobber (PedGeometry* geom)
 {
 	PedFileSystem*		fs;
-	SwapSpecific*		swap_info;
 	char			buf[512];
 
 	fs = swap_open (geom);
 	if (!fs)
 		return 1;
-	swap_info = (SwapSpecific*) fs->type_specific;
 
 	memset (buf, 0, 512);
 	if (!ped_geometry_write (geom, buf, getpagesize() / 512 - 1, 1))
@@ -153,12 +123,31 @@ error_close_fs:
 error:
 	return 0;
 }
+#endif /* !DISCOVER_ONLY */
+
+static void
+swap_init (PedFileSystem* fs, int fresh)
+{
+	SwapSpecific*	fs_info = SWAP_SPECIFIC (fs);
+
+	fs_info->page_sectors = getpagesize () / 512;
+	fs_info->page_count = fs->geom->length / fs_info->page_sectors;
+	fs_info->version = 1;
+	fs_info->max_bad_pages = (getpagesize()
+					- sizeof (SwapNewHeader)) / 4;
+
+	if (fresh)
+		memset (fs_info->header, 0, getpagesize());
+	else
+		ped_geometry_read (fs->geom, fs_info->header,
+				   0, fs_info->page_sectors);
+}
 
 static PedFileSystem*
-swap_alloc (const PedGeometry* geom)
+swap_alloc (PedGeometry* geom)
 {
-	PedFileSystem*		fs;
-	SwapSpecific*		fs_info;
+	PedFileSystem*	fs;
+	SwapSpecific*	fs_info;
 
 	fs = (PedFileSystem*) ped_malloc (sizeof (PedFileSystem));
 	if (!fs)
@@ -182,9 +171,7 @@ swap_alloc (const PedGeometry* geom)
 	fs->geom = ped_geometry_duplicate (geom);
 	if (!fs->geom)
 		goto error_free_buffer;
-
 	fs->type = &swap_type;
-
 	return fs;
 
 error_free_buffer:
@@ -213,36 +200,42 @@ swap_free (PedFileSystem* fs)
 }
 
 static PedFileSystem*
-swap_open (const PedGeometry* geom)
+swap_open (PedGeometry* geom)
 {
 	PedFileSystem*		fs;
 	SwapSpecific*		fs_info;
+	const char*		sig;
 
 	fs = swap_alloc (geom);
 	if (!fs)
 		goto error;
+	swap_init (fs, 0);
 
 	fs_info = SWAP_SPECIFIC (fs);
-	fs_info->page_sectors = getpagesize () / 512;
 	if (!ped_geometry_read (fs->geom, fs_info->header, 0,
 				fs_info->page_sectors))
 		goto error_free_fs;
-	fs_info->sig = ((char*) fs_info->header) + getpagesize() - 10;
 
-	if (strncmp (fs_info->sig, "SWAP-SPACE", 10) == 0) {
+	sig = ((char*) fs_info->header) + getpagesize() - 10;
+	if (strncmp (sig, "SWAP-SPACE", 10) == 0) {
 		fs_info->version = 0;
-	} else if (strncmp (fs_info->sig, "SWAPSPACE2", 10) == 0) {
+		fs_info->page_count
+			= PED_MIN (fs->geom->length / fs_info->page_sectors,
+				   8 * (getpagesize() - 10));
+	} else if (strncmp (sig, "SWAPSPACE2", 10) == 0) {
 		fs_info->version = 1;
+		fs_info->page_count = fs_info->header->new.last_page;
 	} else {
-		char	sig [11];
+		char	_sig [11];
 
-		memcpy (sig, fs_info->sig, 10);
-		sig [10] = 0;
-		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
-			_("Unrecognised linux swap signature '%10s'."), sig);
-		goto error_free_fs;
+		memcpy (_sig, sig, 10);
+		_sig [10] = 0;
+ 		ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
+			_("Unrecognised linux swap signature '%10s'."), _sig);
+ 		goto error_free_fs;
 	}
 
+	fs->checked = 1;
 	return fs;
 
 error_free_fs:
@@ -251,6 +244,14 @@ error:
 	return NULL;
 }
 
+static int
+swap_close (PedFileSystem* fs)
+{
+	swap_free (fs);
+	return 1;
+}
+
+#ifndef DISCOVER_ONLY
 static int
 swap_new_find_bad_page (PedFileSystem* fs, unsigned int page)
 {
@@ -337,16 +338,23 @@ swap_clear_pages (PedFileSystem* fs)
 }
 
 static int
-swap_check_pages (PedFileSystem* fs)
+swap_check_pages (PedFileSystem* fs, PedTimer* timer)
 {
 	SwapSpecific*	fs_info = SWAP_SPECIFIC (fs);
 	PedSector	result;
 	int		first_page = 1;
 	int		stop_page = 0;
 	int		last_page = fs_info->page_count - 1;
+	PedTimer*	nested_timer;
+
+	ped_timer_reset (timer);
+	ped_timer_set_state_name (timer, _("checking for bad blocks"));
 
 	swap_clear_pages (fs);
 	while (first_page <= last_page) {
+		nested_timer = ped_timer_new_nested (
+				  timer,
+			       	  1.0 * (last_page - first_page) / last_page);
 		result = ped_geometry_check (
 				fs->geom,
 				fs_info->buffer,
@@ -354,7 +362,9 @@ swap_check_pages (PedFileSystem* fs)
 				first_page * fs_info->page_sectors,
 				fs_info->page_sectors,
 				(last_page - first_page + 1)
-					* fs_info->page_sectors);
+					* fs_info->page_sectors,
+				nested_timer);
+		ped_timer_destroy_nested (nested_timer);
 		if (!result)
 			return 1;
 		stop_page = result / fs_info->page_sectors;
@@ -365,61 +375,35 @@ swap_check_pages (PedFileSystem* fs)
 	return 1;
 }
 
-/* if rewrite is set, then no warning messages about compatibility are
- * thrown, and the boot-code for new Linux swap partitions is preserved
- * (for LILO and friends).
- */
 static int
-swap_write (PedFileSystem* fs, int rewrite, PedGeometry* old_geom)
+swap_write (PedFileSystem* fs)
 {
 	SwapSpecific*	fs_info = SWAP_SPECIFIC (fs);
+	char*		sig = ((char*) fs_info->header) + getpagesize() - 10;
 
-	fs_info->page_sectors = getpagesize () / 512;
-	fs_info->sig = (char*) fs_info->header
-			+ fs_info->page_sectors * 512 - 10;
-
-	fs_info->page_count = fs->geom->length / fs_info->page_sectors;
-
-	if (!ped_geometry_read (fs->geom, fs_info->header, 0, 2))
-		return 0;
-	if (!rewrite) {
-/* not many people use 2.0 nowadays.  If they are, then there's no harm done...
- * they'll just get a bit confused when they can't swap on...
- */
-#if 0
-		ped_exception_throw (PED_EXCEPTION_WARNING,
-			PED_EXCEPTION_OK,
-			_("This swap partition is not compatible with "
-			"Linux version 2.1.117 or earlier.  Use a "
-			"smaller partition (maximum size 128mb) if you "
-			"want to use old versions of Linux."));
-#endif
-		memset ((char*) fs_info->header + 1024, 0,
-			fs_info->page_sectors * 512 - 1024);
-
-		fs_info->version = 1;
+	if (fs_info->version == 0) {
+		memcpy (sig, "SWAP-SPACE", 10);
+	} else {
 		fs_info->header->new.version = 1;
 		fs_info->header->new.last_page = fs_info->page_count - 1;
 		fs_info->header->new.nr_badpages = 0;
-		fs_info->max_bad_pages = (getpagesize()
-						- sizeof (SwapNewHeader)) / 4;
-		memcpy (fs_info->sig, "SWAPSPACE2", 10);
+		memcpy (sig, "SWAPSPACE2", 10);
 	}
-	if (!swap_check_pages (fs))
-		return 0;
+
 	return ped_geometry_write (fs->geom, fs_info->header, 0,
 				   fs_info->page_sectors);
 }
 
 static PedFileSystem*
-swap_create (PedGeometry* geom)
+swap_create (PedGeometry* geom, PedTimer* timer)
 {
 	PedFileSystem*		fs;
 
 	fs = swap_alloc (geom);
 	if (!fs)
 		goto error;
-	if (!swap_write (fs, 0, NULL))
+	swap_init (fs, 1);
+	if (!swap_write (fs))
 		goto error_free_fs;
 	return fs;
 
@@ -430,20 +414,13 @@ error:
 }
 
 static int
-swap_close (PedFileSystem* fs)
+swap_resize (PedFileSystem* fs, PedGeometry* geom, PedTimer* timer)
 {
-	swap_free (fs);
-	return 1;
-}
+	PedGeometry*	old_geom = fs->geom;
 
-static int
-swap_resize (PedFileSystem* fs, const PedGeometry* geom)
-{
-	PedGeometry*	old_geom;
-	
-	old_geom = fs->geom;
 	fs->geom = ped_geometry_duplicate (geom);
-	if (!swap_write (fs, 0, old_geom))
+	swap_init (fs, old_geom->start != geom->start);
+	if (!swap_write (fs))
 		goto error;
 	ped_geometry_destroy (old_geom);
 	return 1;
@@ -454,133 +431,87 @@ error:
 	return 0;
 }
 
-static int
-swap_copy (const PedFileSystem* fs, PedGeometry* geom)
+static PedFileSystem*
+swap_copy (const PedFileSystem* fs, PedGeometry* geom, PedTimer* timer)
 {
-	PedFileSystem*	new_fs;
-
-	new_fs = swap_create (geom);
-	if (!new_fs)
-		goto error;
-	swap_close (new_fs);
-	return 1;
-
-error:
-	return 0;
+	return ped_file_system_create (geom, &swap_type, timer);
 }
 
 static int
-swap_check (PedFileSystem* fs)
+swap_check (PedFileSystem* fs, PedTimer* timer)
 {
-	swap_write (fs, 1, fs->geom);
-	return 1;
+	return swap_check_pages (fs, timer)
+		&& swap_write (fs);
+}
+
+static PedConstraint*
+swap_get_create_constraint (const PedDevice* dev)
+{
+	PedGeometry	full_dev;
+
+	if (!ped_geometry_init (&full_dev, dev, 0, dev->length - 1))
+		return NULL;
+	
+	return ped_constraint_new (ped_alignment_any, ped_alignment_any,
+				   &full_dev, &full_dev,
+				   getpagesize() / 512, dev->length);
 }
 
 static PedConstraint*
 swap_get_resize_constraint (const PedFileSystem* fs)
 {
-	PedDisk*	disk = fs->geom->disk;
-	PedGeometry	full_disk;
-
-	if (!ped_geometry_init (&full_disk, disk, 0, disk->dev->length - 1))
-		return NULL;
-	
-	return ped_constraint_new (ped_alignment_any, ped_alignment_any,
-				   &full_disk, &full_disk, getpagesize() / 512);
+	return swap_get_create_constraint (fs->geom->dev);
 }
 
-static int
-swap_set_system (const PedFileSystem* fs, PedPartition* part,
-		 const PedDiskType* disk_type)
+static PedConstraint*
+swap_get_copy_constraint (const PedFileSystem* fs, const PedDevice* dev)
 {
-	if (strcasecmp (disk_type->name, LOOP_NAME) == 0)
-		return 1;
+	return swap_get_create_constraint (dev);
+}
+#endif /* !DISCOVER_ONLY */
 
-	if (strcasecmp (disk_type->name, DOS_NAME) == 0) {
-		DosPartitionData*	dos_data = part->disk_specific;
+static PedFileSystemOps swap_ops = {
+	probe:		swap_probe,
+#ifndef DISCOVER_ONLY
+	clobber:	swap_clobber,
+	open:		swap_open,
+	create:		swap_create,
+	close:		swap_close,
+	check:		swap_check,
+	copy:		swap_copy,
+	resize:		swap_resize,
+	get_create_constraint:	swap_get_create_constraint,
+	get_resize_constraint:	swap_get_resize_constraint,
+	get_copy_constraint:	swap_get_copy_constraint
+#else
+	clobber:	NULL,
+	open:		NULL,
+	create:		NULL,
+	close:		NULL,
+	check:		NULL,
+	copy:		NULL,
+	resize:		NULL,
+	get_create_constraint:	NULL,
+	get_resize_constraint:	NULL,
+	get_copy_constraint:	NULL
+#endif /* !DISCOVER_ONLY */
+};
 
-		PED_ASSERT (dos_data != NULL, return 0);
+static PedFileSystemType swap_type = {
+	next:	NULL,
+	ops:	&swap_ops,
+	name:	"linux-swap"
+};
 
-		if (dos_data->raid) {
-			dos_data->system = 0xfd;
-			return 1;
-		}
-		if (dos_data->lvm) {
-			dos_data->system = 0x8e;
-			return 1;
-		}
-
-		if (dos_data->hidden) {
-			ped_exception_throw (
-				PED_EXCEPTION_ERROR,
-				PED_EXCEPTION_FIX,
-				_("Linux-swap partitions can not be hidden on "
-				  "msdos disk labels."));
-			dos_data->hidden = 0;
-		}
-
-		dos_data->system = 0x82;
-		return 1;
-	}
-
-	if (strcasecmp (disk_type->name, PC98_NAME) == 0) {
-		PC98PartitionData*	pc98_data = part->disk_specific;
-
-		PED_ASSERT (pc98_data != NULL, return 0);
-
-		if (pc98_data->boot) {
-			ped_exception_throw (
-				PED_EXCEPTION_ERROR,
-				PED_EXCEPTION_FIX,
-				_("Linux-swap partitions can not be bootable "
-				  "on pc98 disk labels."));
-			pc98_data->boot = 0;
-		}
-
-		pc98_data->system = 0x20e2;
-		return 1;
-	}
-
-	if (strcmp (disk_type->name, MAC_NAME) == 0) {
-		MacPartitionData*       mac_data = part->disk_specific;
-
-		PED_ASSERT (mac_data != NULL, return 0);
-
-		if (mac_data->is_boot)
-			strcpy (mac_data->system_name, "Apple_Bootstrap");
-		else
-			strcpy (mac_data->system_name, "Apple_UNIX_SVR2");
-
-		if (!mac_data->is_swap) {
-			if (!ped_partition_set_flag (part, PED_PARTITION_SWAP,
-						     1))
-				return 0;
-		}
-		mac_data->status = 0x33;
-		return 1;
-	}
-
-	if (strcmp (disk_type->name, BSD_NAME) == 0) {
-		BSDPartitionData* bsd_data = part->disk_specific;
-		bsd_data->type = 0x1;
-		return 1;
-	}
-
-	if (strcmp (disk_type->name, GPT_NAME) == 0) {
-		GPTPartitionData* gpt_part_data = part->disk_specific;
-		PED_ASSERT(gpt_part_data      != NULL, return 0);
-		PED_ASSERT(gpt_part_data->pte != NULL, return 0);
-		gpt_part_data->pte->PartitionTypeGuid =
-			PARTITION_SWAP_GUID;
-		return 1;
-	}
-
-	if (strcmp (disk_type->name, SUN_NAME) == 0) {
-		SunPartitionData* sun_data = part->disk_specific;
-		sun_data->type = 0x82;
-		return 1;
-	}
-
-	return 0;	
+void
+ped_file_system_linux_swap_init ()
+{
+	ped_file_system_type_register (&swap_type);
+}
+
+void
+ped_file_system_linux_swap_done ()
+{
+	ped_file_system_type_unregister (&swap_type);
 }
 
