@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 
     This is all rather complicated.  There are a few combinations:
 	* shared libraries full support
@@ -100,8 +100,11 @@ FCLASS reiserfs_fs_t* (FPTR reiserfs_fs_create) (dal_t *, dal_t *,
 				          reiserfs_gauge_t *);
 
 FCLASS int (FPTR reiserfs_fs_resize) (reiserfs_fs_t *, blk_t, reiserfs_gauge_t *);
+FCLASS int (FPTR reiserfs_fs_check) (reiserfs_fs_t *, reiserfs_gauge_t *);
+
 FCLASS reiserfs_fs_t *(FPTR reiserfs_fs_copy) (reiserfs_fs_t *, dal_t *,
 					reiserfs_gauge_t *);
+
 FCLASS int (FPTR reiserfs_fs_clobber) (dal_t *);
 FCLASS void (FPTR reiserfs_fs_close) (reiserfs_fs_t *);
 
@@ -383,6 +386,7 @@ static PedConstraint *reiserfs_get_create_constraint(const PedDevice *dev)
 static int reiserfs_check(PedFileSystem *fs, PedTimer *timer)
 {
 	reiserfs_fs_t *fs_info;
+	reiserfs_gauge_t *gauge = NULL;
 
 	PED_ASSERT(fs != NULL, return 0);
 
@@ -391,29 +395,56 @@ static int reiserfs_check(PedFileSystem *fs, PedTimer *timer)
 	if (!reiserfs_fs_is_consistent(fs_info)) {
 		ped_exception_throw(PED_EXCEPTION_ERROR,
 				    PED_EXCEPTION_CANCEL,
-				    _
-				    ("Filesystem isn't in valid state. Possible it is mounted."));
+				    _("The file system is in an invalid "
+				      "state.  Perhaps it is mounted?"));
 		return 0;
 	}
 
 	if (!reiserfs_fs_is_resizeable(fs_info))
 		ped_exception_throw(PED_EXCEPTION_WARNING,
 				    PED_EXCEPTION_IGNORE,
-				    _
-				    ("Filesystem is in old (not resizeable format)"));
+				    _("The file system is in old "
+				      "(unresizeable) format."));
 
 	if (!reiserfs_fs_bitmap_check(fs_info)) {
 		ped_exception_throw(PED_EXCEPTION_ERROR,
 				    PED_EXCEPTION_CANCEL,
-				    _
-				    ("Invalid free blocks count. Run reiserfsck --check first."));
+				    _("Invalid free blocks count.  Run "
+				      "reiserfsck --check first."));
 		return 0;
 	}
 
+#ifdef HAVE_REISERFS_FS_CHECK
+	ped_timer_reset(timer);
+	
+	if (libreiserfs_gauge_create && libreiserfs_gauge_free) {
+		if (!
+		    (gauge =
+		     libreiserfs_gauge_create(NULL, gauge_handler, timer)))
+			return 0;
+	}
+		
+	ped_timer_set_state_name(timer, _("checking"));
+	ped_timer_update(timer, 0.0);
+
+	if (!reiserfs_fs_check(fs_info, gauge)) {
+		ped_exception_throw(PED_EXCEPTION_ERROR,
+				    PED_EXCEPTION_CANCEL,
+				    _("Reiserfs tree seems to be corrupted.  "
+				      "Run reiserfsck --check first."));
+		return 0;
+	}
+	
+	ped_timer_update(timer, 1.0);
+
+	if (gauge)
+		libreiserfs_gauge_free(gauge);
+#endif
+	
 	ped_exception_throw(PED_EXCEPTION_INFORMATION, PED_EXCEPTION_OK,
-			    _
-			    ("The reiserfs filesystem passed a basic check.  For a more "
-			     "comprehensive check, use the reiserfsck --check"));
+			    _("The reiserfs file system passed a basic check.  "
+			      "For a more comprehensive check, run "
+			      "reiserfsck --check."));
 
 	return 1;
 }
@@ -431,19 +462,13 @@ static int reiserfs_resize(PedFileSystem *fs, PedGeometry *geom,
 
 	old_length = fs->geom->length;
 
-	if (fs->geom->dev != geom->dev) {
-		ped_exception_throw(PED_EXCEPTION_ERROR,
-				    PED_EXCEPTION_CANCEL,
-				    _
-				    ("Filesystem device isn't equals geometry device."));
-		return 0;
-	}
+	PED_ASSERT (fs->geom->dev == geom->dev, return 0);
 
 	if (fs->geom->start != geom->start) {
 		ped_exception_throw(PED_EXCEPTION_ERROR,
 				    PED_EXCEPTION_CANCEL,
-				    _
-				    ("Sorry, can't move the start of reiserfs partitions yet."));
+				    _("Sorry, can't move the start of "
+				      "reiserfs partitions yet."));
 		return 0;
 	}
 
@@ -458,8 +483,9 @@ static int reiserfs_resize(PedFileSystem *fs, PedGeometry *geom,
 		if (!geom_dal_reopen(dal, O_RDWR)) {
 			ped_exception_throw(PED_EXCEPTION_ERROR,
 					    PED_EXCEPTION_CANCEL,
-					    _
-					    ("Couldn't reopen device abstraction layer for read/write."));
+					    _("Couldn't reopen device "
+					      "abstraction layer for "
+					      "read/write."));
 			return 0;
 		}
 	}
@@ -560,8 +586,8 @@ static PedFileSystem *reiserfs_copy(const PedFileSystem *fs,
 	if (!(dal = geom_dal_create(fs_geom, DEFAULT_BLOCK_SIZE, O_RDWR))) {
 		ped_exception_throw(PED_EXCEPTION_ERROR,
 				    PED_EXCEPTION_CANCEL,
-				    _
-				    ("Couldn't create reiserfs device abstraction handler"));
+				    _("Couldn't create reiserfs device "
+				      "abstraction handler."));
 		goto error_free_fs_geom;
 	}
 
@@ -574,8 +600,7 @@ static PedFileSystem *reiserfs_copy(const PedFileSystem *fs,
 	if (fs_len <= min_needed_blk) {
 		ped_exception_throw(PED_EXCEPTION_ERROR,
 				    PED_EXCEPTION_CANCEL,
-				    _
-				    ("Device is too small for %lu blocks"),
+				    _("Device is too small for %lu blocks."),
 				    min_needed_blk);
 		goto error_free_dal;
 	}
@@ -652,7 +677,8 @@ static void *getsym(void *handle, const char *symbol)
 	if ((error = dlerror()) != NULL) {
 		ped_exception_throw(PED_EXCEPTION_WARNING,
 				    PED_EXCEPTION_IGNORE,
-				    _("Couldn't resolve symbol %s. Error: %s."),
+				    _("Couldn't resolve symbol %s.  "
+				      "Error: %s."),
 				    symbol, error);
 		return NULL;
 	}
@@ -685,7 +711,7 @@ static int reiserfs_ops_interface_version_check(void)
 		ped_exception_throw(
 			PED_EXCEPTION_WARNING, PED_EXCEPTION_CANCEL,
 			_("GNU Parted has detected libreiserfs interface "
-			  "version mismatch. Found %d-%d, required %d. "
+			  "version mismatch.  Found %d-%d, required %d. "
 			  "ReiserFS support will be disabled."),
 			min_interface_version,
 			max_interface_version,
