@@ -1,6 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1999, 2000 Free Software Foundation, Inc.
+    Copyright (C) 1999, 2000, 2005 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
 #include "config.h"
@@ -60,6 +60,11 @@ error:
 	return NULL;
 }
 
+/* This function constructs a PedGeometry object that is an identical but
+ * independent copy of geom.  Both the input, geom, and the output
+ * should be destroyed with ped_geometry_destroy() when they are no
+ * longer needed.
+ */
 PedGeometry*
 ped_geometry_duplicate (const PedGeometry* geom)
 {
@@ -67,6 +72,11 @@ ped_geometry_duplicate (const PedGeometry* geom)
 	return ped_geometry_new (geom->dev, geom->start, geom->length);
 }
 
+/* This function constructs a PedGeometry object that describes the
+ * region that is common to both a and b.  If there is no such common
+ * region, it returns NULL.  (This situation is not treated as an
+ * error by much of Parted.)
+ */
 PedGeometry*
 ped_geometry_intersect (const PedGeometry* a, const PedGeometry* b)
 {
@@ -120,18 +130,21 @@ ped_geometry_set (PedGeometry* geom, PedSector start, PedSector length)
 	return 1;
 }
 
+/* Modifies the start of geom while keeping the end fixed. */
 int
 ped_geometry_set_start (PedGeometry* geom, PedSector start)
 {
 	return ped_geometry_set (geom, start, geom->end - start + 1);
 }
 
+/* Modifies the end of geom while keeping the start fixed. */
 int
 ped_geometry_set_end (PedGeometry* geom, PedSector end)
 {
 	return ped_geometry_set (geom, geom->start, end - geom->start + 1);
 }
 
+/* Returns 1 if the regions that a and b describe overlap. */
 int
 ped_geometry_test_overlap (const PedGeometry* a, const PedGeometry* b)
 {
@@ -147,6 +160,7 @@ ped_geometry_test_overlap (const PedGeometry* a, const PedGeometry* b)
 		return b->end >= a->start;
 }
 
+/* Returns 1 if the region b describes is contained entirely inside a. */
 int
 ped_geometry_test_inside (const PedGeometry* a, const PedGeometry* b)
 {
@@ -159,6 +173,7 @@ ped_geometry_test_inside (const PedGeometry* a, const PedGeometry* b)
 	return b->start >= a->start && b->end <= a->end;
 }
 
+/* Returns 1 if a and b describe the same regions. */
 int
 ped_geometry_test_equal (const PedGeometry* a, const PedGeometry* b)
 {
@@ -170,6 +185,7 @@ ped_geometry_test_equal (const PedGeometry* a, const PedGeometry* b)
 	       && a->end == b->end;
 }
 
+/* Returns 1 if sector lies within the region that geom describes. */
 int
 ped_geometry_test_sector_inside (const PedGeometry* geom, PedSector sector)
 {
@@ -196,7 +212,7 @@ ped_geometry_read (const PedGeometry* geom, void* buffer, PedSector start,
 		exception_status = ped_exception_throw (
 			PED_EXCEPTION_ERROR, PED_EXCEPTION_IGNORE_CANCEL,
 			_("Attempt to read sectors %ld-%ld outside of "
-			  "partition on %s"),
+			  "partition on %s."),
 			(long) start, (long) (start + count - 1),
 			geom->dev->path);
 		return exception_status == PED_EXCEPTION_IGNORE;
@@ -207,6 +223,10 @@ ped_geometry_read (const PedGeometry* geom, void* buffer, PedSector start,
 	return 1;
 }
 
+/* This function flushes all write-behind caches that might be holding
+ * writes made by ped_geometry_write() to geom.  It is slow, because
+ * it guarantees cache coherency among all relevant caches.
+ */
 int
 ped_geometry_sync (PedGeometry* geom)
 {
@@ -214,6 +234,18 @@ ped_geometry_sync (PedGeometry* geom)
 	return ped_device_sync (geom->dev);
 }
 
+/* This function flushes all write-behind caches that might be holding writes
+ * made by ped_geometry_write() to geom.  It does NOT ensure cache coherency
+ * with other caches that cache data in the region described by geom.  If you
+ * need cache coherency, use ped_geometry_sync() instead.
+ */
+int
+ped_geometry_sync_fast (PedGeometry* geom)
+{
+	PED_ASSERT (geom != NULL, return 0);
+	return ped_device_sync_fast (geom->dev);
+}
+
 int
 ped_geometry_write (PedGeometry* geom, const void* buffer, PedSector start,
 		    PedSector count)
@@ -233,7 +265,7 @@ ped_geometry_write (PedGeometry* geom, const void* buffer, PedSector start,
 			PED_EXCEPTION_ERROR,
 			PED_EXCEPTION_IGNORE_CANCEL,
 			_("Attempt to write sectors %ld-%ld outside of "
-			  "partition on %s"),
+			  "partition on %s."),
 			(long) start, (long) (start + count - 1),
 			geom->dev->path);
 		return exception_status == PED_EXCEPTION_IGNORE;
@@ -286,6 +318,18 @@ found_error:
 	goto retry;   /* weird: failure on group read, but not individually */
 }
 
+/* This function takes a sector inside the region described by src, and
+ * returns that sector's address inside dst.  This means that
+ *
+ * 	ped_geometry_read (dst, buf, ped_geometry_map(dst, src, sector), 1)
+ *
+ * does the same thing as
+ * 
+ * 	ped_geometry_read (src, buf, sector, 1)
+ *
+ * Clearly, this will only work if src and dst overlap.  If the sector
+ * is not contained in dst, then this function returns -1.
+ */
 PedSector
 ped_geometry_map (const PedGeometry* dst, const PedGeometry* src,
 		  PedSector sector)
@@ -295,6 +339,8 @@ ped_geometry_map (const PedGeometry* dst, const PedGeometry* src,
 	PED_ASSERT (dst != NULL, return 0);
 	PED_ASSERT (src != NULL, return 0);
 
+	if (!ped_geometry_test_sector_inside (src, sector))
+		return -1;
 	if (dst->dev != src->dev)
 		return -1;
 
