@@ -1,6 +1,6 @@
 /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
+    Copyright (C) 1999, 2000, 2001, 2005 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
 #define _GNU_SOURCE 1
@@ -352,8 +352,8 @@ gnu_close (PedDevice* dev)
 		ped_exception_throw (
 			PED_EXCEPTION_WARNING,
 			PED_EXCEPTION_OK,
-			_("The partition table was unable to be reread, "
-			  "as such, you need to reboot before mounting any "
+			_("The partition table cannot be re-read.  This means "
+			  "you need to reboot before mounting any "
 			  "modified partitions.  You also need to reinstall "
 			  "your boot loader before you reboot (which may "
 			  "require mounting modified partitions).  It is "
@@ -369,10 +369,10 @@ gnu_close (PedDevice* dev)
 		ped_exception_throw (
 			PED_EXCEPTION_WARNING,
 			PED_EXCEPTION_IGNORE,
-			_("The partition table on %s (%s).  This means "
-			  "the Hurd knows nothing about any modifications "
-			  "you made.  You should reboot your computer "
-			  "before doing anything with %s."),
+			_("The partition table on %s cannot be re-read "
+			  "(%s).  This means the Hurd knows nothing about any "
+			  "modifications you made.  You should reboot your "
+			  "computer before doing anything with %s."),
 			dev->path, strerror (errno), dev->path);
 	}
 
@@ -860,6 +860,7 @@ static PedDeviceArchOps gnu_dev_ops = {
 	write:		gnu_write,
 	check:		gnu_check,
 	sync:		gnu_sync,
+	sync_fast:	gnu_sync,
 	probe_all:	gnu_probe_all
 };
 
