@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
 #include "config.h"
@@ -53,7 +53,7 @@ typedef struct
 } pointer_size_type;
 
 #ifdef DEBUG
-static pointer_size_type dodgey_malloc_list[] = {
+static pointer_size_type dodgy_malloc_list[] = {
  {0,		0},
  {0,		0},
  {0,		0},
@@ -66,7 +66,7 @@ static pointer_size_type dodgey_malloc_list[] = {
  {0,		0}
 };
 
-static int	dodgey_memory_active[100];
+static int	dodgy_memory_active[100];
 #endif /* DEBUG */
 
 int
@@ -79,13 +79,14 @@ ped_set_architecture (const PedArchitecture* arch)
 }
 
 extern void ped_disk_bsd_init ();
+extern void ped_disk_dvh_init ();
 extern void ped_disk_gpt_init ();
 extern void ped_disk_loop_init ();
 extern void ped_disk_mac_init ();
-extern void ped_disk_dvh_init ();
 extern void ped_disk_msdos_init ();
 extern void ped_disk_pc98_init ();
 extern void ped_disk_sun_init ();
+extern void ped_disk_amiga_init ();
 
 static void
 init_disk_types ()
@@ -97,13 +98,15 @@ init_disk_types ()
 	ped_disk_pc98_init ();
 #endif
 	ped_disk_msdos_init ();
-	ped_disk_dvh_init ();
 	ped_disk_mac_init ();
 	ped_disk_gpt_init ();
+	ped_disk_dvh_init ();
 	ped_disk_bsd_init ();
+	ped_disk_amiga_init ();
 }
 
 #ifdef ENABLE_FS
+extern void ped_file_system_amiga_init (void);
 extern void ped_file_system_xfs_init (void);
 extern void ped_file_system_ufs_init (void);
 extern void ped_file_system_reiserfs_init (void);
@@ -117,6 +120,7 @@ extern void ped_file_system_ext2_init (void);
 static void
 init_file_system_types ()
 {
+	ped_file_system_amiga_init ();
 	ped_file_system_xfs_init ();
 	ped_file_system_ufs_init ();
 	ped_file_system_reiserfs_init ();
@@ -130,13 +134,14 @@ init_file_system_types ()
 #endif /* ENABLE_FS */
 
 extern void ped_disk_bsd_done ();
+extern void ped_disk_dvh_done ();
 extern void ped_disk_gpt_done ();
 extern void ped_disk_loop_done ();
 extern void ped_disk_mac_done ();
-extern void ped_disk_dvh_done ();
 extern void ped_disk_msdos_done ();
 extern void ped_disk_pc98_done ();
 extern void ped_disk_sun_done ();
+extern void ped_disk_amiga_done ();
 
 static void
 done_disk_types ()
@@ -146,11 +151,12 @@ done_disk_types ()
 	ped_disk_pc98_done ();
 #endif
 	ped_disk_msdos_done ();
-	ped_disk_dvh_done ();
 	ped_disk_mac_done ();
 	ped_disk_loop_done ();
 	ped_disk_gpt_done ();
+	ped_disk_dvh_done ();
 	ped_disk_bsd_done ();
+	ped_disk_amiga_done ();
 }
 
 static void _init() __attribute__ ((constructor));
@@ -176,7 +182,7 @@ _init()
 #endif
 
 #ifdef DEBUG
-	memset (dodgey_memory_active, 0, sizeof (dodgey_memory_active));
+	memset (dodgy_memory_active, 0, sizeof (dodgy_memory_active));
 #endif
 }
 
@@ -190,6 +196,7 @@ extern void ped_file_system_ntfs_done (void);
 extern void ped_file_system_reiserfs_done (void);
 extern void ped_file_system_ufs_done (void);
 extern void ped_file_system_xfs_done (void);
+extern void ped_file_system_amiga_done (void);
 
 static void
 done_file_system_types ()
@@ -203,6 +210,7 @@ done_file_system_types ()
 	ped_file_system_reiserfs_done ();
 	ped_file_system_ufs_done ();
 	ped_file_system_xfs_done ();
+	ped_file_system_amiga_done ();
 }
 #endif /* ENABLE_FS */
 
@@ -228,16 +236,16 @@ ped_get_version ()
 
 #ifdef DEBUG
 static void
-_check_dodgey_pointer (const void* ptr, size_t size, int is_malloc)
+_check_dodgy_pointer (const void* ptr, size_t size, int is_malloc)
 {
 	int		i;
 
-	for (i=0; dodgey_malloc_list[i].pointer; i++) {
-		if (dodgey_malloc_list[i].pointer != ptr)
+	for (i=0; dodgy_malloc_list[i].pointer; i++) {
+		if (dodgy_malloc_list[i].pointer != ptr)
  			continue;
-		if (is_malloc && dodgey_malloc_list[i].size != size)
+		if (is_malloc && dodgy_malloc_list[i].size != size)
 			continue;
-		if (!is_malloc && !dodgey_memory_active[i])
+		if (!is_malloc && !dodgy_memory_active[i])
 			continue;
 
 
@@ -245,16 +253,16 @@ _check_dodgey_pointer (const void* ptr, size_t size, int is_malloc)
 			ped_exception_throw (
 				PED_EXCEPTION_INFORMATION,
 				PED_EXCEPTION_OK,
-				"Dodgey malloc(%x) == %p occured (active==%d)",
-				size, ptr, dodgey_memory_active[i]);
-			dodgey_memory_active[i]++;
+				"Dodgy malloc(%x) == %p occurred (active==%d)",
+				size, ptr, dodgy_memory_active[i]);
+			dodgy_memory_active[i]++;
 		} else {
 			ped_exception_throw (
 				PED_EXCEPTION_INFORMATION,
 				PED_EXCEPTION_OK,
-				"Dodgey free(%p) occured (active==%d)",
-				ptr, dodgey_memory_active[i]);
-			dodgey_memory_active[i]--;
+				"Dodgy free(%p) occurred (active==%d)",
+				ptr, dodgy_memory_active[i]);
+			dodgy_memory_active[i]--;
 		}
 
 		return;
@@ -276,7 +284,7 @@ ped_malloc (size_t size)
 
 #ifdef DEBUG
 	memset (mem, 0xff, size);
-	_check_dodgey_pointer (mem, size, 1);
+	_check_dodgy_pointer (mem, size, 1);
 #endif
 
 	return mem;
@@ -301,7 +309,7 @@ void
 ped_free (void* ptr)
 {
 #ifdef DEBUG
-	_check_dodgey_pointer (ptr, 0, 0);
+	_check_dodgy_pointer (ptr, 0, 0);
 #endif
 
 	free (ptr);
