@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
 #include "command.h"
@@ -71,7 +71,7 @@ command_get (Command** list, char* name)
 		return NULL;
 
 	for (i=0; list [i]; i++) {
-		switch (str_list_match (list [i]->names, name)) {
+		switch (str_list_match_any (list [i]->names, name)) {
 		case 2:
 			return list [i];
 
@@ -93,6 +93,18 @@ command_get (Command** list, char* name)
 		return list [partial_match];
 }
 
+StrList*
+command_get_names (Command** list)
+{
+	Command**	walk;
+	StrList*	result = NULL;
+
+	for (walk = list; *walk; walk++)
+		result = str_list_join (result,
+					str_list_duplicate ((*walk)->names));
+	return result;
+}
+
 void
 command_print_summary (Command* cmd)
 {
