@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
 #ifndef COMMAND_H_INCLUDED
@@ -38,6 +38,7 @@ extern void command_destroy (Command* cmd);
 void command_register (Command** list, Command* cmd);
 
 extern Command* command_get (Command** list, char* name);
+extern StrList* command_get_names (Command** list);
 extern void command_print_summary (Command* cmd);
 extern void command_print_help (Command* cmd);
 extern int command_run (Command* cmd, PedDevice** dev);
