@@ -1,6 +1,6 @@
 /*
     parted - a frontend to libparted
-    Copyright (C) 1999, 2000 Free Software Foundation, Inc.
+    Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
 */
 
 #ifndef STRLIST_H_INCLUDED
@@ -32,22 +32,27 @@ extern char* language;
 extern char* gettext_charset;
 extern char* term_charset;
 
-extern int init_str_list ();
-extern int done_str_list ();
-
 extern StrList* str_list_create (const char* first, ...);
 extern StrList* str_list_create_unique (const char* first, ...);
 extern void str_list_destroy (StrList* list);
+extern void str_list_destroy_node (StrList* list);
 
+extern StrList* str_list_duplicate (const StrList* list);
+extern StrList* str_list_duplicate_node (const StrList* list);
+extern StrList* str_list_insert (StrList* list, const char* str);
 extern StrList* str_list_append (StrList* list, const char* str);
 extern StrList* str_list_append_unique (StrList* list, const char* str);
+extern StrList* str_list_join (StrList* a, StrList* b);
 extern char* str_list_convert (const StrList* list);
 extern char* str_list_convert_node (const StrList* list);
 
 extern void str_list_print (const StrList* list);
 extern void str_list_print_wrap (const StrList* list, int line_length,
 				 int offset, int indent);
-extern int str_list_match (StrList* list, char* str);
-extern int str_list_match_node (StrList* list, char* str);
+extern int str_list_match_any (const StrList* list, const char* str);
+extern int str_list_match_node (const StrList* list, const char* str);
+extern StrList* str_list_match (const StrList* list, const char* str);
+
+extern int str_list_length (const StrList* list);
 
 #endif /* STRLIST_H_INCLUDED */
